=== modified file 'client/drizzletest.cc'
--- client/drizzletest.cc	2011-02-24 13:59:28 +0000
+++ client/drizzletest.cc	2011-02-28 14:09:50 +0000
@@ -83,7 +83,6 @@
 using namespace std;
 using namespace drizzled;
 
-extern "C"
 unsigned char *get_var_key(const unsigned char* var, size_t *len, bool);
 
 int get_one_option(int optid, const struct option *, char *argument);
@@ -6195,17 +6194,30 @@
 
 /* Definitions for replace result */
 
-typedef struct st_pointer_array {    /* when using array-strings */
+class POINTER_ARRAY
+{    /* when using array-strings */
+public:
+  ~POINTER_ARRAY();
+  int insert(char* name);
+
+  POINTER_ARRAY()
+  {
+    memset(this, 0, sizeof(*this));
+  }
+
   TYPELIB typelib;        /* Pointer to strings */
-  unsigned char  *str;          /* Strings is here */
-  uint8_t *flag;          /* Flag about each var. */
-  uint32_t  array_allocs,max_count,length,max_length;
-} POINTER_ARRAY;
+  unsigned char *str;          /* Strings is here */
+  uint8_t* flag;          /* Flag about each var. */
+  uint32_t array_allocs;
+  uint32_t max_count;
+  uint32_t length;
+  uint32_t max_length;
+};
 
 struct st_replace;
 struct st_replace *init_replace(const char **from, const char **to, uint32_t count,
                                 char *word_end_chars);
-int insert_pointer_name(POINTER_ARRAY *pa,char * name);
+
 void replace_strings_append(struct st_replace *rep, string* ds,
                             const char *from, int len);
 
@@ -6220,54 +6232,47 @@
   variable is replaced.
 */
 
-static void free_pointer_array(POINTER_ARRAY *pa)
+POINTER_ARRAY::~POINTER_ARRAY()
 {
-  if (!pa->typelib.count)
+  if (!typelib.count)
     return;
-  pa->typelib.count=0;
-  free((char*) pa->typelib.type_names);
-  pa->typelib.type_names=0;
-  free(pa->str);
+  typelib.count= 0;
+  free((char*) typelib.type_names);
+  typelib.type_names=0;
+  free(str);
 }
 
-void do_get_replace(struct st_command *command)
+void do_get_replace(st_command *command)
 {
-  uint32_t i;
   char *from= command->first_argument;
-  char *buff, *start;
-  char word_end_chars[256], *pos;
+  if (!*from)
+    die("Missing argument in %s", command->query);
+  free_replace();
   POINTER_ARRAY to_array, from_array;
-
-
-  free_replace();
-
-  memset(&to_array, 0, sizeof(to_array));
-  memset(&from_array, 0, sizeof(from_array));
-  if (!*from)
-    die("Missing argument in %s", command->query);
-  start= buff= (char *)malloc(strlen(from)+1);
+  char* start= (char*)malloc(strlen(from) + 1);
+  char* buff= start;
   while (*from)
   {
-    char *to= buff;
-    to= get_string(&buff, &from, command);
+    char *to= get_string(&buff, &from, command);
     if (!*from)
-      die("Wrong number of arguments to replace_result in '%s'",
-          command->query);
-    insert_pointer_name(&from_array,to);
+      die("Wrong number of arguments to replace_result in '%s'", command->query);
+    from_array.insert(to);
     to= get_string(&buff, &from, command);
-    insert_pointer_name(&to_array,to);
+    to_array.insert(to);
   }
-  for (i= 1,pos= word_end_chars ; i < 256 ; i++)
-    if (my_isspace(charset_info,i))
+  char word_end_chars[256];
+  char* pos= word_end_chars;
+  for (int i= 1; i < 256; i++)
+  {
+    if (my_isspace(charset_info, i))
       *pos++= i;
+  }
   *pos=0;          /* End pointer */
   if (!(glob_replace= init_replace(from_array.typelib.type_names,
                                    to_array.typelib.type_names,
                                    from_array.typelib.count,
                                    word_end_chars)))
     die("Can't initialize replace from '%s'", command->query);
-  free_pointer_array(&from_array);
-  free_pointer_array(&to_array);
   free(start);
   command->last_argument= command->end;
   return;
@@ -7177,7 +7182,7 @@
 #define PC_MALLOC    256  /* Bytes for pointers */
 #define PS_MALLOC    512  /* Bytes for data */
 
-int insert_pointer_name(POINTER_ARRAY *pa,char * name)
+static int insert_pointer_name(POINTER_ARRAY* pa, char* name)
 {
   uint32_t i,length,old_count;
   unsigned char *new_pos;
@@ -7245,6 +7250,11 @@
   return(0);
 } /* insert_pointer_name */
 
+int POINTER_ARRAY::insert(char* name)
+{
+  return insert_pointer_name(this, name);
+}
+
 
 /* Functions that uses replace and replace_regex */
 

=== modified file 'docs/boolean_data_type.rst'
--- docs/boolean_data_type.rst	2011-02-23 14:17:23 +0000
+++ docs/boolean_data_type.rst	2011-02-27 02:23:00 +0000
@@ -12,6 +12,4 @@
 A value of ``FALSE`` can be assigned to a ``BOOLEAN`` column by either assigning
 ``0``, ``FALSE``, ``"FALSE"``, ``"F"``, or ``"NO"``.
 
-Please note that using ``BOOL`` as a data type is different from using
-``BOOLEAN`` as a type. ``BOOL`` creates an ``INTEGER`` column. This is done in
-order to keep compatiblity with MySQL.
+The default value for a ``BOOLEAN`` column can only be ``TRUE`` or ``FALSE``.

=== modified file 'docs/include.am'
--- docs/include.am	2011-02-24 14:52:15 +0000
+++ docs/include.am	2011-02-25 17:02:30 +0000
@@ -14,6 +14,7 @@
 	     docs/plugins/innobase/index.rst \
 	     docs/plugins/mysql_protocol/index.rst \
 	     docs/plugins/transaction_log/index.rst \
+	     docs/plugins/slave/index.rst \
 	     docs/_static/drizzle.ico \
 	     ${top_srcdir}/docs/*.rst \
 	     ${top_srcdir}/docs/clients/*.rst \

=== modified file 'docs/index.rst'
--- docs/index.rst	2011-02-24 14:52:15 +0000
+++ docs/index.rst	2011-02-25 01:07:38 +0000
@@ -133,6 +133,7 @@
  
    testing/test-run.rst
    testing/dbqp.rst
+   testing/randgen.rst
 
 Indices and tables
 ==================

=== modified file 'docs/testing/dbqp.rst'
--- docs/testing/dbqp.rst	2011-02-23 23:29:48 +0000
+++ docs/testing/dbqp.rst	2011-02-24 23:19:35 +0000
@@ -1,9 +1,10 @@
 **********************************
-dbqp.py - Drizzle testing tool 
+dbqp
 **********************************
 
 Synopsis
 ========
+Drizzle testing tool
 
 **./dbqp** [ *OPTIONS* ] [ TESTCASE ]
 

=== added file 'docs/testing/randgen.rst'
--- docs/testing/randgen.rst	1970-01-01 00:00:00 +0000
+++ docs/testing/randgen.rst	2011-02-24 23:19:35 +0000
@@ -0,0 +1,197 @@
+**********************************
+randgen (random query generator)
+**********************************
+
+
+
+Description
+===========
+
+The randgen aka the random query generator is a database
+testing tool.  It uses a grammar-based stochastic model to represent
+some desired set of queries (to exercise the optimizer, for example) 
+and generates random queries as allowed by the grammar
+
+The primary documentation is here: http://forge.mysql.com/wiki/RandomQueryGenerator
+
+This document is intended to help the user set up their environment so that the tool
+may be used in conjunction with the dbqp.py test-runner.  The forge documentation
+contains more information on the particulars of the tool itself.
+
+Requirements
+============
+
+DBD::drizzle
+-------------
+The DBD::drizzle module is required it can be found here http://launchpad.net/dbd-drizzle/
+
+Additional information for installing the module::
+
+    Prerequisites
+    ----------------
+    * Perl
+    * Drizzle (bzr branch lp:drizzle)
+    * libdrizzle (bzr branch lp:libdrizzle)
+    * C compiler
+
+    Installation
+    -------------
+    You should only have to run the following:
+
+    perl Makefile.PL --cflags=-I/usr/local/drizzle/include/ --libs=-"L/usr/local/drizzle/lib -ldrizzle"
+
+
+    Depending on where libdrizzle is installed. Also, you'll want to make 
+    sure that ldconfig has configured libdrizzle to be in your library path 
+
+Additional information may be found here: http://forge.mysql.com/wiki/RandomQueryGeneratorQuickStart
+
+Installing the randgen
+=======================
+
+The code may be branched from launchpad: bzr branch lp:randgen
+
+it also may be downloaded from here http://launchpad.net/randgen/+download
+
+That is all there is : )
+
+Randgen / dbqp tests
+====================
+
+These tests are simple .cnf files that can define a few basic variables
+that are needed to execute tests.  The most interesting section is test_servers.  It is a simple list of python lists
+Each sub-list contains a string of server options that are needed.  Each sub-list represents a server that will be started.
+Using an empty sub-list will create a server with the default options::
+
+    [test_info]
+    comment = does NOT actually test the master-slave replication yet, but it will.
+
+    [test_command]
+    command = ./gentest.pl --gendata=conf/drizzle/drizzle.zz --grammar=conf/drizzle/optimizer_subquery_drizzle.yy --queries=10 --threads=1
+
+    [test_servers]
+    servers = [[--innodb.replication-log=true],[--plugin-add=slave --slave.config-file=$MASTER_SERVER_SLAVE_CONFIG]]
+
+Running tests
+=========================
+
+There are several different ways to run tests using :doc:`dbqp` 's randgen mode.
+
+It should be noted that unless :option:`--force` is used, the program will
+stop execution upon encountering the first failing test. 
+:option:`--force` is recommended if you are running several tests - it will
+allow you to view all successes and failures in one run.
+
+Running individual tests
+------------------------
+If one only wants to run a few, specific tests, they may do so this way::
+
+    ./dbqp --mode=randgen --randgen-path=/path/to/randgen [OPTIONS] test1 [test2 ... testN]
+
+Running all tests within a suite
+--------------------------------
+Many of the tests supplied with Drizzle are organized into suites.  
+
+The tests within drizzle/tests/randgen_tests/main are considered the 'main' suite.  
+Other suites are also subdirectories of drizzle/tests/randgen_tests.
+
+To run the tests in a specific suite::
+
+    ./dbqp --mode=randgen --randgen-path=/path/to/randgen [OPTIONS] --suite=SUITENAME
+
+Running specific tests within a suite
+--------------------------------------
+To run a specific set of tests within a suite::
+
+    ./dbqp --mode=randgen --randgen-path=/path/to/randgen [OPTIONS] --suite=SUITENAME TEST1 [TEST2..TESTN]
+
+Calling tests using <suitename>.<testname> currently does not work.
+One must specify the test suite via the :option:`--suite` option.
+
+
+Running all available tests
+---------------------------
+One would currently have to name all suites, but the majority of the working tests live in the main suite
+Other suites utilize more exotic server combinations and we are currently tweaking them to better integrate with the 
+dbqp system.  The slave-plugin suite does currently have a good config file for setting up simple replication setups for testing.
+To execute several suites' worth of tests::
+
+    ./dbqp --mode=randgen --randgen-path=/path/to/randgen [OPTIONS] --suite=SUITE1, SUITE2, ...SUITEN
+
+Interpreting test results
+=========================
+The output of the test runner is quite simple.  Every test should pass.
+In the event of a test failure, please take the time to file a bug here:
+*https://bugs.launchpad.net/drizzle*
+
+During a run, the program will provide the user with:
+  * test name (suite + name)
+  * test status (pass/fail/skipped)
+  * time spent executing each test
+
+At the end of a run, the program will provide the user with a listing of:
+  * how many tests were run
+  * how many tests failed
+  * percentage of passing tests
+  * a listing of failing tests
+  * total time spent executing the tests
+
+Example output::
+
+    24 Feb 2011 17:27:36 : main.outer_join_portable                                [ pass ]         7019
+    24 Feb 2011 17:27:39 : main.repeatable_read                                    [ pass ]         2764
+    24 Feb 2011 17:28:57 : main.select_stability_validator                         [ pass ]        77946
+    24 Feb 2011 17:29:01 : main.subquery                                           [ pass ]         4474
+    24 Feb 2011 17:30:52 : main.subquery_semijoin                                  [ pass ]       110355
+    24 Feb 2011 17:31:00 : main.subquery_semijoin_nested                           [ pass ]         8750
+    24 Feb 2011 17:31:03 : main.varchar                                            [ pass ]         3048
+    24 Feb 2011 17:31:03 : ================================================================================
+    24 Feb 2011 17:31:03 INFO: Test execution complete in 288 seconds
+    24 Feb 2011 17:31:03 INFO: Summary report:
+    24 Feb 2011 17:31:03 INFO: Executed 18/18 test cases, 100.00 percent
+    24 Feb 2011 17:31:03 INFO: STATUS: PASS, 18/18 test cases, 100.00 percent executed
+    24 Feb 2011 17:31:03 INFO: Spent 287 / 288 seconds on: TEST(s)
+    24 Feb 2011 17:31:03 INFO: Test execution complete
+    24 Feb 2011 17:31:03 INFO: Stopping all running servers...
+
+    
+Additional uses
+===============
+Starting a server for manual testing and (optionally) populating it
+--------------------------------------------------------------------
+
+:doc:`dbqp` 's randgen mode allows a user to get a Drizzle server up and running quickly.  This can be useful for fast ad-hoc testing.
+
+To do so call::
+
+    ./dbqp --mode=randgen --randgen-path=/path/to/randgen --start-and-exit [*OPTIONS*]
+
+This will start a Drizzle server that you can connect to and query
+
+With the addition of the --gendata option, a user may utilize the randgen's gendata (table creation and population) tool
+to populate a test server.  In the following example, the test server is now populated by the 8 tables listed below::
+
+    ./dbqp --mode=randgen --randgen-path=/randgen --start-and-exit --gendata=/randgen/conf/drizzle/drizzle.zz
+    <snip>
+    24 Feb 2011 17:48:48 INFO: NAME: server0
+    24 Feb 2011 17:48:48 INFO: MASTER_PORT: 9306
+    24 Feb 2011 17:48:48 INFO: DRIZZLE_TCP_PORT: 9307
+    24 Feb 2011 17:48:48 INFO: MC_PORT: 9308
+    24 Feb 2011 17:48:48 INFO: PBMS_PORT: 9309
+    24 Feb 2011 17:48:48 INFO: RABBITMQ_NODE_PORT: 9310
+    24 Feb 2011 17:48:48 INFO: VARDIR: /home/pcrews/bzr/work/dbqp_randgen_updates/tests/workdir/testbot0/server0/var
+    24 Feb 2011 17:48:48 INFO: STATUS: 1
+    # 2011-02-24T17:48:48 Default schema: test
+    # 2011-02-24T17:48:48 Executor initialized, id GenTest::Executor::Drizzle 2011.02.2198 ()
+    # 2011-02-24T17:48:48 # Creating Drizzle table: test.A; engine: ; rows: 0 .
+    # 2011-02-24T17:48:48 # Creating Drizzle table: test.B; engine: ; rows: 0 .
+    # 2011-02-24T17:48:48 # Creating Drizzle table: test.C; engine: ; rows: 1 .
+    # 2011-02-24T17:48:48 # Creating Drizzle table: test.D; engine: ; rows: 1 .
+    # 2011-02-24T17:48:48 # Creating Drizzle table: test.AA; engine: ; rows: 10 .
+    # 2011-02-24T17:48:48 # Creating Drizzle table: test.BB; engine: ; rows: 10 .
+    # 2011-02-24T17:48:48 # Creating Drizzle table: test.CC; engine: ; rows: 100 .
+    # 2011-02-24T17:48:49 # Creating Drizzle table: test.DD; engine: ; rows: 100 .
+    24 Feb 2011 17:48:49 INFO: User specified --start-and-exit.  dbqp.py exiting and leaving servers running...
+
+
+

=== modified file 'drizzled/cached_item.cc'
--- drizzled/cached_item.cc	2011-02-17 00:14:13 +0000
+++ drizzled/cached_item.cc	2011-02-24 19:13:33 +0000
@@ -28,6 +28,7 @@
 #include <drizzled/cached_item.h>
 #include <drizzled/sql_string.h>
 #include <drizzled/session.h>
+#include <drizzled/item/field.h>
 #include <algorithm>
 
 using namespace std;

=== modified file 'drizzled/catalog/cache.cc'
--- drizzled/catalog/cache.cc	2011-02-24 12:29:34 +0000
+++ drizzled/catalog/cache.cc	2011-02-26 16:03:36 +0000
@@ -29,10 +29,10 @@
 Instance::shared_ptr Cache::find(const identifier::Catalog &identifier, drizzled::error_t &error)
 {
   boost::mutex::scoped_lock scopedLock(_mutex);
-  if (const unordered_map::mapped_type* i= find_ptr(cache, identifier))
+  if (const unordered_map::mapped_type* ptr= find_ptr(cache, identifier))
   {
-    error= *i ? EE_OK : ER_CATALOG_NO_LOCK;
-    return *i;
+    error= *ptr ? EE_OK : ER_CATALOG_NO_LOCK;
+    return *ptr;
   }
   error= ER_CATALOG_DOES_NOT_EXIST;
   return catalog::Instance::shared_ptr();
@@ -60,9 +60,9 @@
 bool Cache::unlock(const identifier::Catalog &identifier, drizzled::error_t &error)
 {
   boost::mutex::scoped_lock scopedLock(_mutex);
-  if (const unordered_map::mapped_type* i= find_ptr(cache, identifier))
+  if (const unordered_map::mapped_type* ptr= find_ptr(cache, identifier))
   {
-    if (not *i)
+    if (not *ptr)
     {
       if (cache.erase(identifier))
         return true;

=== modified file 'drizzled/catalog/local.h'
--- drizzled/catalog/local.h	2011-02-17 00:14:13 +0000
+++ drizzled/catalog/local.h	2011-02-18 16:08:59 +0000
@@ -24,14 +24,15 @@
 
 #include <drizzled/identifier.h>
 #include <drizzled/catalog/instance.h>
+#include <drizzled/visibility.h>
 
 namespace drizzled
 {
 namespace catalog
 {
 
-identifier::Catalog::const_reference local_identifier();
-Instance::shared_ptr local();
+DRIZZLED_API identifier::Catalog::const_reference local_identifier();
+DRIZZLED_API Instance::shared_ptr local();
 
 } /* namespace catalog */
 } /* namespace drizzled */

=== modified file 'drizzled/definition/cache.cc'
--- drizzled/definition/cache.cc	2011-02-24 12:41:15 +0000
+++ drizzled/definition/cache.cc	2011-02-26 16:03:36 +0000
@@ -37,8 +37,8 @@
 table::instance::Shared::shared_ptr Cache::find(const identifier::Table::Key &key)
 {
   boost::mutex::scoped_lock scopedLock(_mutex);
-  if (Map::mapped_type* i= find_ptr(cache, key))
-    return *i;
+  if (Map::mapped_type* ptr= find_ptr(cache, key))
+    return *ptr;
   return table::instance::Shared::shared_ptr();
 }
 

=== modified file 'drizzled/errmsg_print.h'
--- drizzled/errmsg_print.h	2011-02-17 00:14:13 +0000
+++ drizzled/errmsg_print.h	2011-02-23 20:25:56 +0000
@@ -38,9 +38,9 @@
 DRIZZLED_API bool errmsg_printf(error::level_t priority, char const *format, ...)
   __attribute__((format(printf, 2, 3)));
 
-void sql_perror(const char *message);
-void sql_perror(const std::string &message);
-void sql_perror(std::string message, const std::string &extra);
+DRIZZLED_API void sql_perror(const char *message);
+DRIZZLED_API void sql_perror(const std::string &message);
+DRIZZLED_API void sql_perror(std::string message, const std::string &extra);
 
 } /* namespace drizzled */
 

=== modified file 'drizzled/error.cc'
--- drizzled/error.cc	2011-02-24 14:51:20 +0000
+++ drizzled/error.cc	2011-02-28 14:09:50 +0000
@@ -677,7 +677,7 @@
   ADD_ERROR_MESSAGE(ER_WRONG_NAME_FOR_CATALOG, N_("Invalid catalog name."));
   ADD_ERROR_MESSAGE(ER_USE_DATA_DICTIONARY, N_("Engine status is now stored in the data_dictionary tables, please use these instead."));
   ADD_ERROR_MESSAGE(ER_TRANSACTION_ALREADY_STARTED, N_("There is already a transaction in progress"));
-
+  ADD_ERROR_MESSAGE(ER_NO_LOCK_HELD, N_("No lock is held by this connection."));
 }
 
 } /* namespace drizzled */

=== modified file 'drizzled/error_t.h'
--- drizzled/error_t.h	2011-02-15 02:11:09 +0000
+++ drizzled/error_t.h	2011-02-24 07:46:16 +0000
@@ -865,7 +865,8 @@
   ER_TRANSACTIONAL_DDL_NOT_SUPPORTED,
   ER_USE_DATA_DICTIONARY,
   ER_TRANSACTION_ALREADY_STARTED,
-  ER_CARTESIAN_JOIN_ATTEMPTED
+  ER_CARTESIAN_JOIN_ATTEMPTED,
+  ER_NO_LOCK_HELD
 };
 
 

=== modified file 'drizzled/execute.cc'
--- drizzled/execute.cc	2011-02-17 00:14:13 +0000
+++ drizzled/execute.cc	2011-02-18 16:08:59 +0000
@@ -22,6 +22,7 @@
 
 #include <drizzled/session.h>
 #include <drizzled/user_var_entry.h>
+#include <drizzled/plugin/client/cached.h>
 #include <drizzled/plugin/client/concurrent.h>
 #include <drizzled/catalog/local.h>
 #include <drizzled/execute.h>
@@ -45,6 +46,67 @@
   run(execution_string);
 }
 
+void Execute::run(std::string &execution_string, sql::ResultSet &result_set)
+{
+  boost_thread_shared_ptr thread;
+  
+  if (_session.isConcurrentExecuteAllowed())
+  {
+    plugin::client::Cached *client= new plugin::client::Cached(result_set);
+    client->pushSQL(execution_string);
+    Session::shared_ptr new_session= Session::make_shared(client, catalog::local());
+    
+    // We set the current schema.  @todo do the same with catalog
+    util::string::const_shared_ptr schema(_session.schema());
+    if (not schema->empty())
+      new_session->set_db(*schema);
+    
+    new_session->setConcurrentExecute(false);
+    
+    // Overwrite the context in the next session, with what we have in our
+    // session. Eventually we will allow someone to change the effective
+    // user.
+    new_session->user()= _session.user();
+    
+    if (Session::schedule(new_session))
+    {
+      Session::unlink(new_session);
+    }
+    else if (wait)
+    {
+      thread= new_session->getThread();
+    }
+  }
+  else
+  {
+    my_error(ER_WRONG_ARGUMENTS, MYF(0), "A Concurrent Execution Session can not launch another session.");
+    return;
+  }
+  
+  if (wait && thread && thread->joinable())
+  {
+    // We want to make sure that we can be killed
+    if (_session.getThread())
+    {
+      boost::this_thread::restore_interruption dl(_session.getThreadInterupt());
+      
+      try {
+        thread->join();
+      }
+      catch(boost::thread_interrupted const&)
+      {
+        // Just surpress and return the error
+        my_error(drizzled::ER_QUERY_INTERRUPTED, MYF(0));
+        return;
+      }
+    }
+    else
+    {
+      thread->join();
+    }
+  }
+}
+
 void Execute::run(std::string &execution_string)
 {
   boost_thread_shared_ptr thread;

=== modified file 'drizzled/execute.h'
--- drizzled/execute.h	2010-12-08 03:38:36 +0000
+++ drizzled/execute.h	2011-02-18 20:43:32 +0000
@@ -21,10 +21,17 @@
 #ifndef DRIZZLED_EXECUTE_H
 #define DRIZZLED_EXECUTE_H
 
+#include <drizzled/visibility.h>
+
 namespace drizzled
 {
 
-class Execute
+namespace sql
+{
+  class ResultSet;
+}
+
+class DRIZZLED_API Execute
 {
   bool wait;
   Session &_session;
@@ -35,6 +42,7 @@
 
   void run(std::string &to_execute);
   void run(const char *arg, size_t length);
+  void run(std::string &execution_string, sql::ResultSet &result_set);
 
   Session &session()
   {

=== modified file 'drizzled/field_iterator.h'
--- drizzled/field_iterator.h	2011-02-19 23:46:11 +0000
+++ drizzled/field_iterator.h	2011-02-24 19:13:33 +0000
@@ -24,9 +24,9 @@
 #include <drizzled/memory/sql_alloc.h>
 #include <drizzled/sql_list.h>
 #include <drizzled/natural_join_column.h>
+#include <drizzled/item/field.h>
 
-namespace drizzled
-{
+namespace drizzled {
 
 class Table;
 class TableList;

=== modified file 'drizzled/filesort.cc'
--- drizzled/filesort.cc	2011-02-18 22:30:53 +0000
+++ drizzled/filesort.cc	2011-02-24 19:13:33 +0000
@@ -46,14 +46,12 @@
 #include <drizzled/plugin/transactional_storage_engine.h>
 #include <drizzled/atomics.h>
 #include <drizzled/global_buffer.h>
-
 #include <drizzled/sort_field.h>
-
+#include <drizzled/item/subselect.h>
 
 using namespace std;
 
-namespace drizzled
-{
+namespace drizzled {
 
 /* Defines used by filesort and uniques */
 #define MERGEBUFF		7

=== modified file 'drizzled/include.am'
--- drizzled/include.am	2011-02-24 05:43:31 +0000
+++ drizzled/include.am	2011-02-24 16:51:37 +0000
@@ -352,6 +352,7 @@
 			      drizzled/plugin/authorization.h \
 			      drizzled/plugin/catalog.h \
 			      drizzled/plugin/client.h \
+			      drizzled/plugin/client/cached.h \
 			      drizzled/plugin/client/concurrent.h \
 			      drizzled/plugin/daemon.h \
 			      drizzled/plugin/error_message.h \
@@ -410,6 +411,9 @@
 			      drizzled/show.h \
 			      drizzled/show_type.h \
 			      drizzled/signal_handler.h \
+			      drizzled/sql/exception.h \
+			      drizzled/sql/result_set.h \
+			      drizzled/sql/result_set_meta_data.h \
 			      drizzled/sort_field.h \
 			      drizzled/sql_base.h \
 			      drizzled/sql_error.h \
@@ -813,6 +817,8 @@
 			   drizzled/set_var.cc \
 			   drizzled/show.cc \
 			   drizzled/signal_handler.cc \
+			   drizzled/sql/exception.cc \
+			   drizzled/sql/result_set.cc \
 			   drizzled/sql_base.cc \
 			   drizzled/sql_delete.cc \
 			   drizzled/sql_derived.cc \

=== modified file 'drizzled/item.cc'
--- drizzled/item.cc	2011-02-23 00:13:15 +0000
+++ drizzled/item.cc	2011-02-24 19:13:33 +0000
@@ -31,10 +31,8 @@
 #include <drizzled/util/convert.h>
 #include <drizzled/plugin/client.h>
 #include <drizzled/time_functions.h>
-
 #include <drizzled/field/str.h>
 #include <drizzled/field/num.h>
-
 #include <drizzled/field/blob.h>
 #include <drizzled/field/date.h>
 #include <drizzled/field/datetime.h>
@@ -50,11 +48,11 @@
 #include <drizzled/field/size.h>
 #include <drizzled/field/time.h>
 #include <drizzled/field/varstring.h>
-
 #include <drizzled/current_session.h>
 #include <drizzled/session.h>
-
 #include <drizzled/internal/m_string.h>
+#include <drizzled/item/ref.h>
+#include <drizzled/item/subselect.h>
 
 #include <cstdio>
 #include <math.h>

=== modified file 'drizzled/item/create.cc'
--- drizzled/item/create.cc	2011-02-23 00:13:15 +0000
+++ drizzled/item/create.cc	2011-02-27 02:10:05 +0000
@@ -1070,7 +1070,7 @@
 Item*
 Create_udf_func::create(Session *session, LEX_STRING name, List<Item> *item_list)
 {
-  const plugin::Function *udf= plugin::Function::get(name.str, name.length);
+  const plugin::Function *udf= plugin::Function::get(std::string(name.str, name.length));
   assert(udf);
   return create(session, udf, item_list);
 }

=== modified file 'drizzled/item/field.cc'
--- drizzled/item/field.cc	2011-02-23 00:13:15 +0000
+++ drizzled/item/field.cc	2011-02-24 19:13:33 +0000
@@ -29,6 +29,7 @@
 #include <drizzled/item/field.h>
 #include <drizzled/item/outer_ref.h>
 #include <drizzled/plugin/client.h>
+#include <drizzled/item/subselect.h>
 
 #include <boost/dynamic_bitset.hpp>
 

=== modified file 'drizzled/item/ref.cc'
--- drizzled/item/ref.cc	2011-02-21 18:33:47 +0000
+++ drizzled/item/ref.cc	2011-02-24 19:13:33 +0000
@@ -25,6 +25,7 @@
 #include <drizzled/item/ref.h>
 #include <drizzled/plugin/client.h>
 #include <drizzled/item/sum.h>
+#include <drizzled/item/subselect.h>
 
 namespace drizzled
 {

=== modified file 'drizzled/item/sum.cc'
--- drizzled/item/sum.cc	2011-02-23 00:13:15 +0000
+++ drizzled/item/sum.cc	2011-02-24 19:13:33 +0000
@@ -30,7 +30,6 @@
 #include <drizzled/hybrid_type_traits_decimal.h>
 #include <drizzled/sql_base.h>
 #include <drizzled/session.h>
-
 #include <drizzled/item/sum.h>
 #include <drizzled/field/decimal.h>
 #include <drizzled/field/double.h>
@@ -38,10 +37,9 @@
 #include <drizzled/field/date.h>
 #include <drizzled/field/datetime.h>
 #include <drizzled/unique.h>
-
 #include <drizzled/type/decimal.h>
-
 #include <drizzled/internal/m_string.h>
+#include <drizzled/item/subselect.h>
 
 #include <algorithm>
 

=== modified file 'drizzled/join.cc'
--- drizzled/join.cc	2011-02-25 11:05:55 +0000
+++ drizzled/join.cc	2011-02-28 14:09:50 +0000
@@ -61,8 +61,8 @@
 #include <drizzled/plugin/storage_engine.h>
 #include <drizzled/session.h>
 #include <drizzled/select_result.h>
-
 #include <drizzled/debug.h>
+#include <drizzled/item/subselect.h>
 
 #include <algorithm>
 

=== modified file 'drizzled/message/cache.cc'
--- drizzled/message/cache.cc	2011-02-24 12:53:01 +0000
+++ drizzled/message/cache.cc	2011-02-26 16:03:36 +0000
@@ -29,8 +29,8 @@
 table::shared_ptr Cache::find(const identifier::Table &identifier)
 {
   boost_unique_lock_t scoped_lock(_access);
-  if (Map::mapped_type* i= find_ptr(cache, identifier.getKey()))
-    return *i;
+  if (Map::mapped_type* ptr= find_ptr(cache, identifier.getKey()))
+    return *ptr;
   return table::shared_ptr();
 }
 

=== modified file 'drizzled/optimizer/explain_plan.h'
--- drizzled/optimizer/explain_plan.h	2010-12-18 04:43:40 +0000
+++ drizzled/optimizer/explain_plan.h	2011-02-24 19:13:33 +0000
@@ -20,9 +20,9 @@
 #ifndef DRIZZLED_OPTIMIZER_EXPLAIN_PLAN_H
 #define DRIZZLED_OPTIMIZER_EXPLAIN_PLAN_H
 
-namespace drizzled
-{
+namespace drizzled {
 
+class Join;
 class Session;
 class Select_Lex_Unit;
 class select_result;

=== modified file 'drizzled/optimizer/key_field.cc'
--- drizzled/optimizer/key_field.cc	2011-02-22 23:20:59 +0000
+++ drizzled/optimizer/key_field.cc	2011-02-24 19:13:33 +0000
@@ -27,6 +27,7 @@
 #include <drizzled/optimizer/key_field.h>
 #include <drizzled/optimizer/key_use.h>
 #include <drizzled/sql_lex.h>
+#include <drizzled/item/subselect.h>
 
 #include <vector>
 

=== modified file 'drizzled/parser.cc'
--- drizzled/parser.cc	2011-02-24 18:16:42 +0000
+++ drizzled/parser.cc	2011-02-27 02:20:23 +0000
@@ -22,6 +22,7 @@
 #include <drizzled/parser.h>
 #include <drizzled/alter_info.h>
 #include <drizzled/alter_drop.h>
+#include <drizzled/item/subselect.h>
 
 namespace drizzled {
 namespace parser {
@@ -178,17 +179,16 @@
 
 Item* reserved_keyword_function(Session *session, const std::string &name, List<Item> *item_list)
 {
-  const plugin::Function *udf= plugin::Function::get(name.c_str(), name.length());
-  Item *item= NULL;
+  const plugin::Function *udf= plugin::Function::get(name);
 
   if (udf)
   {
-    item= Create_udf_func::s_singleton.create(session, udf, item_list);
-  } else {
-    my_error(ER_SP_DOES_NOT_EXIST, MYF(0), "FUNCTION", name.c_str());
+    return Create_udf_func::s_singleton.create(session, udf, item_list);
   }
 
-  return item;
+  my_error(ER_SP_DOES_NOT_EXIST, MYF(0), "FUNCTION", name.c_str());
+
+  return NULL;
 }
 
 /**

=== modified file 'drizzled/parser.h'
--- drizzled/parser.h	2011-02-23 23:48:30 +0000
+++ drizzled/parser.h	2011-02-24 20:50:45 +0000
@@ -19,11 +19,13 @@
 #include <drizzled/charset.h>
 #include <drizzled/error.h>
 #include <drizzled/foreign_key.h>
+#include <drizzled/function/bit/functions.h>
 #include <drizzled/function/get_system_var.h>
 #include <drizzled/function/locate.h>
 #include <drizzled/function/set_user_var.h>
 #include <drizzled/function/str/char.h>
 #include <drizzled/function/str/collation.h>
+#include <drizzled/function/str/concat.h>
 #include <drizzled/function/str/insert.h>
 #include <drizzled/function/str/left.h>
 #include <drizzled/function/str/repeat.h>
@@ -46,7 +48,6 @@
 #include <drizzled/function/time/timestamp_diff.h>
 #include <drizzled/function/time/typecast.h>
 #include <drizzled/function/time/year.h>
-#include <drizzled/function/bit/functions.h>
 #include <drizzled/global_charset_info.h>
 #include <drizzled/internal/m_string.h>
 #include <drizzled/item/boolean.h>

=== added file 'drizzled/plugin/client/cached.h'
--- drizzled/plugin/client/cached.h	1970-01-01 00:00:00 +0000
+++ drizzled/plugin/client/cached.h	2011-02-23 15:58:52 +0000
@@ -0,0 +1,179 @@
+/* - mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2011 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+
+#ifndef DRIZZLED_PLUGIN_CLIENT_CACHED_H
+#define DRIZZLED_PLUGIN_CLIENT_CACHED_H
+
+
+#include <drizzled/plugin/client/concurrent.h>
+#include <drizzled/sql/result_set.h>
+
+#include <boost/scoped_ptr.hpp>
+
+#include <iostream>
+
+namespace drizzled
+{
+namespace plugin
+{
+namespace client
+{
+
+class Cached : public Concurrent
+{
+  uint32_t column;
+  uint32_t max_column;
+  sql::ResultSet *_result_set;
+
+public:
+  Cached(sql::ResultSet &rs) :
+    Concurrent(),
+    column(0),
+    max_column(0),
+    _result_set(&rs)
+  {
+  }
+
+  virtual bool sendFields(List<Item> *list)
+  {
+    List<Item>::iterator it(list->begin());
+    Item *item;
+
+    column= 0;
+    max_column= 0;
+
+    while ((item=it++))
+    {
+      SendField field;
+      item->make_field(&field);
+      max_column++;
+    }
+    _result_set->createRow();
+
+    return false;
+  }
+
+  virtual void sendError(drizzled::error_t error_code, const char *error_message)
+  {
+    sql::Exception tmp(error_message, error_code);
+    _result_set->pushException(tmp);
+  }
+
+  virtual void checkRowEnd(void)
+  {
+    if (++column % max_column == 0)
+    {
+      _result_set->createRow();
+    }
+  }
+
+  using Client::store;
+
+  virtual bool store(Field *from)
+  {
+    if (from->is_null())
+      return store();
+
+    char buff[MAX_FIELD_WIDTH];
+    String str(buff, sizeof(buff), &my_charset_bin);
+    from->val_str_internal(&str);
+
+    return store(str.ptr(), str.length());
+  }
+
+  virtual bool store(void)
+  {
+    _result_set->setColumnNull(currentColumn());
+
+    checkRowEnd();
+
+    return false;
+  }
+
+  virtual bool store(int32_t from)
+  {
+    std::string tmp;
+
+    tmp= boost::lexical_cast<std::string>(from);
+    _result_set->setColumn(currentColumn(), tmp);
+    checkRowEnd();
+
+    return false;
+  }
+
+  virtual bool store(uint32_t from)
+  {
+    std::string tmp;
+
+    tmp= boost::lexical_cast<std::string>(from);
+    _result_set->setColumn(currentColumn(), tmp);
+    checkRowEnd();
+
+    return false;
+  }
+
+  virtual bool store(int64_t from)
+  {
+    std::string tmp;
+
+    tmp= boost::lexical_cast<std::string>(from);
+    _result_set->setColumn(currentColumn(), tmp);
+    checkRowEnd();
+
+    return false;
+  }
+
+  virtual bool store(uint64_t from)
+  {
+    std::string tmp;
+
+    tmp= boost::lexical_cast<std::string>(from);
+    _result_set->setColumn(currentColumn(), tmp);
+    checkRowEnd();
+
+    return false;
+  }
+
+  virtual bool store(double from, uint32_t decimals, String *buffer)
+  {
+    buffer->set_real(from, decimals, &my_charset_bin);
+    return store(buffer->ptr(), buffer->length());
+  }
+
+  virtual bool store(const char *from, size_t length)
+  {
+    _result_set->setColumn(currentColumn(), std::string(from, length));
+    checkRowEnd();
+
+    return false;
+  }
+  
+  inline uint32_t currentColumn() const
+  {
+    return column % max_column;
+  }
+};
+
+} /* namespace client */
+} /* namespace plugin */
+} /* namespace drizzled */
+
+#endif /* DRIZZLED_PLUGIN_CLIENT_CACHED_H */

=== modified file 'drizzled/plugin/function.cc'
--- drizzled/plugin/function.cc	2011-02-24 13:59:28 +0000
+++ drizzled/plugin/function.cc	2011-02-28 14:09:50 +0000
@@ -59,6 +59,7 @@
 
   std::pair<UdfMap::iterator, bool> ret=
     udf_registry.insert(make_pair(udf->getName(), udf));
+
   if (ret.second == false)
   {
     errmsg_printf(error::ERROR,
@@ -75,10 +76,10 @@
   udf_registry.erase(udf->getName());
 }
 
-const plugin::Function *plugin::Function::get(const char *name, size_t length)
+const plugin::Function *plugin::Function::get(const std::string &name)
 {
-  UdfMap::mapped_type* i= find_ptr(udf_registry, std::string(name, length));
-  return i ? *i : NULL;
+  UdfMap::mapped_type* ptr= find_ptr(udf_registry, name);
+  return ptr ? *ptr : NULL;
 }
 
 } /* namespace drizzled */

=== modified file 'drizzled/plugin/function.h'
--- drizzled/plugin/function.h	2011-02-17 00:14:13 +0000
+++ drizzled/plugin/function.h	2011-02-27 02:10:05 +0000
@@ -80,7 +80,7 @@
    */
   static void removePlugin(const plugin::Function *function_obj);
 
-  static const plugin::Function *get(const char *name, size_t len=0);
+  static const plugin::Function *get(const std::string &name);
 
   typedef boost::unordered_map<std::string, const plugin::Function *, util::insensitive_hash, util::insensitive_equal_to> UdfMap;
   typedef boost::unordered_map<std::string, const plugin::Function *, util::insensitive_hash, util::insensitive_equal_to> Map;

=== modified file 'drizzled/session.cc'
--- drizzled/session.cc	2011-02-24 14:51:20 +0000
+++ drizzled/session.cc	2011-02-28 14:09:50 +0000
@@ -68,6 +68,7 @@
 #include <drizzled/util/functors.h>
 #include <drizzled/util/find_ptr.h>
 #include <plugin/myisam/myisam.h>
+#include <drizzled/item/subselect.h>
 
 #include <algorithm>
 #include <climits>

=== modified file 'drizzled/session.h'
--- drizzled/session.h	2011-02-24 13:59:28 +0000
+++ drizzled/session.h	2011-02-28 14:09:50 +0000
@@ -55,7 +55,6 @@
 #include <drizzled/session/table_messages.h>
 #include <drizzled/session/transactions.h>
 #include <drizzled/sql_error.h>
-#include <drizzled/sql_lex.h>
 #include <drizzled/sql_locale.h>
 #include <drizzled/statistics_variables.h>
 #include <drizzled/system_variables.h>
@@ -66,6 +65,8 @@
 #include <drizzled/var.h>
 #include <drizzled/visibility.h>
 #include <drizzled/util/find_ptr.h>
+#include <drizzled/type/time.h>
+#include <drizzled/sql_lex.h>
 
 #define MIN_HANDSHAKE_SIZE      6
 

=== added directory 'drizzled/sql'
=== added file 'drizzled/sql/exception.cc'
--- drizzled/sql/exception.cc	1970-01-01 00:00:00 +0000
+++ drizzled/sql/exception.cc	2011-02-02 22:48:57 +0000
@@ -0,0 +1,49 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ * Copyright (C) 2011, Brian Aker
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ *   * Redistributions of source code must retain the above copyright notice,
+ *     this list of conditions and the following disclaimer.
+ *   * Redistributions in binary form must reproduce the above copyright notice,
+ *     this list of conditions and the following disclaimer in the documentation
+ *     and/or other materials provided with the distribution.
+ *   * Neither the name of Patrick Galbraith nor the names of its contributors
+ *     may be used to endorse or promote products derived from this software
+ *     without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
+ * THE POSSIBILITY OF SUCH DAMAGE.
+ */
+
+#include "config.h"
+
+#include <drizzled/sql/exception.h>
+
+#include <iostream>
+
+namespace drizzled {
+namespace sql {
+
+std::ostream& operator<<(std::ostream& output, const Exception &arg)
+{
+  output << "Exception() " << arg.getErrorMessage()  << ":" << arg.getErrorCode() << std::endl;
+
+  return output;
+}
+
+} // namespace sql 
+} // namespace drizzled

=== added file 'drizzled/sql/exception.h'
--- drizzled/sql/exception.h	1970-01-01 00:00:00 +0000
+++ drizzled/sql/exception.h	2011-02-02 22:48:57 +0000
@@ -0,0 +1,124 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ * Copyright (C) 2011, Brian Aker
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ *   * Redistributions of source code must retain the above copyright notice,
+ *     this list of conditions and the following disclaimer.
+ *   * Redistributions in binary form must reproduce the above copyright notice,
+ *     this list of conditions and the following disclaimer in the documentation
+ *     and/or other materials provided with the distribution.
+ *   * Neither the name of Patrick Galbraith nor the names of its contributors
+ *     may be used to endorse or promote products derived from this software
+ *     without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
+ * THE POSSIBILITY OF SUCH DAMAGE.
+ */
+
+#ifndef DRIZZLED_SQL_EXCEPTION_H
+#define DRIZZLED_SQL_EXCEPTION_H
+
+#include <drizzled/error_t.h>
+
+#include <queue>
+#include <stdexcept>
+
+namespace drizzled {
+namespace sql {
+
+class ResultSet;
+
+class Exception :  public std::runtime_error
+{
+public:
+  Exception(std::string reason, std::string sql_state, drizzled::error_t error_arg) :
+    std::runtime_error(reason),
+    _error_code(error_arg),
+    _sql_state(sql_state),
+    _reason(reason)
+  {
+    if (_sql_state.length() != 5)
+      _sql_state= "HY000";
+  }
+
+  Exception() :
+    std::runtime_error("no error"),
+    _error_code(drizzled::EE_OK),
+    _sql_state("00000")
+  {
+  }
+
+  Exception(std::string reason, drizzled::error_t error_arg) :
+    std::runtime_error(reason),
+    _error_code(error_arg),
+    _reason(reason)
+  {
+    _sql_state= "00000";
+  }
+
+  ~Exception () throw ()
+  { }
+
+  drizzled::error_t getErrorCode() const
+  {
+    return _error_code;
+  }
+
+  const std::string &getErrorMessage() const
+  {
+    return _reason;
+  }
+
+  Exception getNextException() const
+  {
+    if (_next_exception.empty())
+    {
+      return Exception();
+    }
+
+    Exception tmp= _next_exception.front();
+    _next_exception.pop();
+
+    return Exception();
+  }
+
+  const std::string &getSQLState() const
+  {
+    return _sql_state;
+  }
+
+protected:
+  friend class ResultSet;
+  void setNextException(Exception arg) 
+  {
+    (void)arg;
+    return;
+  }
+
+private:
+  drizzled::error_t _error_code; // Vendor, ie our, code
+  std::string _sql_state;
+  std::string _reason;
+  mutable std::queue <Exception> _next_exception;
+};
+
+std::ostream& operator<<(std::ostream& output, const Exception &arg);
+
+} // namespace sql 
+} // namespace drizzled
+
+#endif /* DRIZZLED_SQL_EXCEPTION_H */

=== added file 'drizzled/sql/result_set.cc'
--- drizzled/sql/result_set.cc	1970-01-01 00:00:00 +0000
+++ drizzled/sql/result_set.cc	2011-02-09 16:20:32 +0000
@@ -0,0 +1,174 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ * Copyright (C) 2011, Brian Aker
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ *   * Redistributions of source code must retain the above copyright notice,
+ *     this list of conditions and the following disclaimer.
+ *   * Redistributions in binary form must reproduce the above copyright notice,
+ *     this list of conditions and the following disclaimer in the documentation
+ *     and/or other materials provided with the distribution.
+ *   * Neither the name of Patrick Galbraith nor the names of its contributors
+ *     may be used to endorse or promote products derived from this software
+ *     without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
+ * THE POSSIBILITY OF SUCH DAMAGE.
+ */
+
+#include "config.h"
+
+#include <drizzled/sql/exception.h>
+#include <drizzled/sql/result_set.h>
+
+#include <iostream>
+
+namespace drizzled {
+namespace sql {
+
+static Exception exception_unknown_column("Unknown Column", "S0022", ER_BAD_FIELD_ERROR);
+static Exception exception_no_more_results("No additional rows founds", "S0022", ER_BAD_FIELD_ERROR);
+
+ResultSet::~ResultSet()
+{
+}
+
+const std::string ResultSet::getString(size_t column_number) const
+{
+  if (not isMore(column_number))
+    return "";
+
+  return (*_current_row)[column_number].value();
+}
+
+bool ResultSet::isNull(size_t column_number) const
+{
+  return (*_current_row)[column_number].isNull();
+}
+
+void ResultSet::pushException(const Exception &arg) const
+{
+  if (_exceptions.empty())
+  {
+    _exceptions.push(arg);
+    return;
+  }
+
+  _exceptions.front().setNextException(arg);
+}
+
+bool ResultSet::isMore() const
+{
+  if (_current_row == _results.end())
+  {
+    pushException(exception_no_more_results);
+    return false;
+  }
+
+  return true;
+}
+
+bool ResultSet::isMore(size_t column_number) const
+{
+  if (column_number >= _meta_data.getColumnCount())
+  {
+    pushException(exception_unknown_column);
+
+    return false;
+  }
+
+  return isMore();
+}
+
+bool ResultSet::error() const
+{
+  return not _exceptions.empty();
+}
+
+sql::Exception ResultSet::getException() const
+{
+  return _exceptions.empty() ? sql::Exception() : _exceptions.front();
+}
+
+const ResultSetMetaData &ResultSet::getMetaData() const
+{
+  return _meta_data;
+}
+
+void ResultSet::createRow()
+{
+  assert(_meta_data.getColumnCount());
+  _results.resize(_results.size() +1);
+  _results.back().resize(_meta_data.getColumnCount());
+}
+
+void ResultSet::setColumn(size_t column_number, const std::string &arg)
+{
+  assert(column_number < _meta_data.getColumnCount());
+  assert(_results.back().at(column_number).isNull() == false); // ie the default value
+  assert(_results.back().at(column_number).value().empty() == true); // ie no value has been set yet
+  _results.back().at(column_number).set_value(arg);
+}
+
+void ResultSet::setColumnNull(size_t column_number)
+{
+  assert(column_number < _meta_data.getColumnCount());
+  assert(_results.back().at(column_number).isNull() == false); // ie the default value
+  assert(_results.back().at(column_number).value().empty() == true); // ie no value has been set yet
+  _results.back().at(column_number).set_null();
+}
+
+bool ResultSet::next() const
+{
+  if (not _has_next_been_called)
+  {
+    _current_row= _results.begin();
+    _has_next_been_called= true;
+  }
+  else
+  {
+    _current_row++;
+  }
+
+  if (_current_row == _results.end())
+    return false;
+
+  return true;
+}
+
+std::ostream& operator<<(std::ostream& output, const ResultSet &result_set)
+{
+  while (result_set.next())
+  {
+    for (size_t x= 0; x < result_set.getMetaData().getColumnCount(); x++)
+    {
+      if (result_set.isNull(x))
+      {
+        output << "<null>" << '\t';
+      }
+      else 
+      {
+        output << result_set.getString(x) << '\t';
+      }
+    }
+    output << std::endl;
+  }
+
+  return output;
+}
+
+} // namespace sql 
+} // namespace drizzled

=== added file 'drizzled/sql/result_set.h'
--- drizzled/sql/result_set.h	1970-01-01 00:00:00 +0000
+++ drizzled/sql/result_set.h	2011-02-11 17:56:48 +0000
@@ -0,0 +1,146 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ * Copyright (C) 2011, Brian Aker
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ *   * Redistributions of source code must retain the above copyright notice,
+ *     this list of conditions and the following disclaimer.
+ *   * Redistributions in binary form must reproduce the above copyright notice,
+ *     this list of conditions and the following disclaimer in the documentation
+ *     and/or other materials provided with the distribution.
+ *   * Neither the name of Patrick Galbraith nor the names of its contributors
+ *     may be used to endorse or promote products derived from this software
+ *     without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
+ * THE POSSIBILITY OF SUCH DAMAGE.
+ */
+
+#ifndef DRIZZLED_SQL_RESULT_SET_H
+#define DRIZZLED_SQL_RESULT_SET_H
+
+#include <drizzled/visibility.h>
+#include <drizzled/sql/exception.h>
+#include <drizzled/sql/result_set_meta_data.h>
+#include <cassert>
+#include <queue>
+
+namespace drizzled {
+
+namespace plugin { class Client; }
+
+namespace sql {
+
+class DRIZZLED_API ResultSet
+{
+  // First version of API stores everything as strings
+  class Column {
+  public:
+    Column() :
+      _is_null(false)
+    { }
+
+    inline const std::string &value() const
+    {
+      return _value;
+    }
+
+    inline bool isNull() const
+    {
+      return _is_null;
+    }
+
+    inline void set_value(const std::string &ref)
+    {
+      _value= ref;
+    }
+
+    inline void set_null()
+    {
+      assert(_value.empty());
+      _is_null= true;
+    }
+
+  private:
+    std::string _value;
+    bool _is_null;
+  };
+
+  typedef std::vector< Column > Row;
+  typedef std::vector< Row > Result;
+
+public:
+  static ResultSet *make(size_t field_count)
+  {
+    return new ResultSet(field_count);
+  }
+
+  // Special case for us (throw is just too damn slow for internal usage, or
+  // almost any).
+  bool isNoThrow() const
+  {
+    return true;
+  }
+
+  bool next() const;
+
+  const std::string getString(size_t column_number) const ;
+  bool isNull(size_t column_number) const;
+  const ResultSetMetaData &getMetaData() const;
+
+  // Our functions to use instead of exceptions
+  bool error() const;
+  sql::Exception getException() const;
+
+#if 0
+protected:
+  friend class Client;
+#endif
+  ResultSet(size_t fields) :
+    _has_next_been_called(false),
+    _current_row(_results.end()),
+    _meta_data(fields)
+  {
+  }
+
+  ~ResultSet();
+
+  void createRow();
+  void setColumn(size_t column_number, const std::string &arg);
+  void setColumnNull(size_t column_number);
+  void pushException(const Exception &arg) const;
+
+private: // Member methods
+  bool isMore() const;
+  bool isMore(size_t column_number) const;
+
+private: // Member variables
+  mutable bool _has_next_been_called;
+  Result _results;
+  mutable Result::const_iterator _current_row;
+  ResultSetMetaData _meta_data;
+  
+  // Because an error could come up during a fetch on const, we need to have
+  // this be mutable.
+  mutable std::queue<Exception> _exceptions;
+};
+
+std::ostream& operator<<(std::ostream& output, const ResultSet &result_set);
+
+} // namespace sql 
+} // namespace drizzled
+
+#endif /* DRIZZLED_SQL_RESULT_SET_H */

=== added file 'drizzled/sql/result_set_meta_data.h'
--- drizzled/sql/result_set_meta_data.h	1970-01-01 00:00:00 +0000
+++ drizzled/sql/result_set_meta_data.h	2011-02-02 22:48:57 +0000
@@ -0,0 +1,70 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ * Copyright (C) 2011, Brian Aker
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ *   * Redistributions of source code must retain the above copyright notice,
+ *     this list of conditions and the following disclaimer.
+ *   * Redistributions in binary form must reproduce the above copyright notice,
+ *     this list of conditions and the following disclaimer in the documentation
+ *     and/or other materials provided with the distribution.
+ *   * Neither the name of Patrick Galbraith nor the names of its contributors
+ *     may be used to endorse or promote products derived from this software
+ *     without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
+ * THE POSSIBILITY OF SUCH DAMAGE.
+ */
+
+#ifndef DRIZZLED_SQL_RESULT_SET_META_DATA_H
+#define DRIZZLED_SQL_RESULT_SET_META_DATA_H
+
+#include <cassert>
+#include <queue>
+
+namespace drizzled {
+namespace sql {
+
+class ResultSet;
+
+class ResultSetMetaData
+{
+public:
+  inline size_t getColumnCount() const
+  {
+    return _columns;
+  }
+
+protected:
+  friend class ResultSet;
+
+  ResultSetMetaData(size_t columns) :
+    _columns(columns)
+  {
+  }
+
+private: // Member methods
+
+private: // Member variables
+  const size_t _columns;
+};
+
+std::ostream& operator<<(std::ostream& output, const ResultSetMetaData &result_set);
+
+} // namespace sql 
+} // namespace drizzled
+
+#endif /* DRIZZLED_SQL_RESULT_SET_META_DATA_H */

=== modified file 'drizzled/sql_base.cc'
--- drizzled/sql_base.cc	2011-02-25 11:05:55 +0000
+++ drizzled/sql_base.cc	2011-02-28 14:09:50 +0000
@@ -58,6 +58,7 @@
 #include <drizzled/table/unused.h>
 #include <drizzled/plugin/storage_engine.h>
 #include <drizzled/session.h>
+#include <drizzled/item/subselect.h>
 
 #include <drizzled/refresh_version.h>
 

=== modified file 'drizzled/sql_lex.cc'
--- drizzled/sql_lex.cc	2011-02-23 12:43:31 +0000
+++ drizzled/sql_lex.cc	2011-02-27 02:20:23 +0000
@@ -30,6 +30,7 @@
 #include <drizzled/lookup_symbol.h>
 #include <drizzled/index_hint.h>
 #include <drizzled/select_result.h>
+#include <drizzled/item/subselect.h>
 
 #include <cstdio>
 #include <ctype.h>
@@ -1061,6 +1062,7 @@
       lip->yyUnget();                   // Safety against eof
       state = MY_LEX_START;		// Try again
       break;
+
     case MY_LEX_LONG_COMMENT:		/* Long C comment? */
       if (lip->yyPeek() != '*')
       {
@@ -1158,6 +1160,7 @@
       lip->in_comment= NO_COMMENT;
       lip->set_echo(true);
       break;
+
     case MY_LEX_END_LONG_COMMENT:
       if ((lip->in_comment != NO_COMMENT) && lip->yyPeek() == '/')
       {
@@ -1174,6 +1177,7 @@
       else
         state=MY_LEX_CHAR;		// Return '*'
       break;
+
     case MY_LEX_SET_VAR:		// Check if ':='
       if (lip->yyPeek() != '=')
       {
@@ -1182,6 +1186,7 @@
       }
       lip->yySkip();
       return (SET_VAR);
+
     case MY_LEX_SEMICOLON:			// optional line terminator
       if (lip->yyPeek())
       {
@@ -1190,6 +1195,7 @@
       }
       lip->next_state=MY_LEX_END;       // Mark for next loop
       return(END_OF_INPUT);
+
     case MY_LEX_EOL:
       if (lip->eof())
       {
@@ -1205,11 +1211,13 @@
       }
       state=MY_LEX_CHAR;
       break;
+
     case MY_LEX_END:
       lip->next_state=MY_LEX_END;
       return false;			// We found end of input last time
 
       /* Actually real shouldn't start with . but allow them anyhow */
+
     case MY_LEX_REAL_OR_POINT:
       if (my_isdigit(cs,lip->yyPeek()))
         state= MY_LEX_REAL;		// Real
@@ -1219,6 +1227,7 @@
         lip->yyUnget();                 // Put back '.'
       }
       break;
+
     case MY_LEX_USER_END:		// end '@' of user@hostname
       switch (state_map[(uint8_t)lip->yyPeek()]) {
       case MY_LEX_STRING:
@@ -1235,12 +1244,14 @@
       yylval->lex_str.str=(char*) lip->get_ptr();
       yylval->lex_str.length=1;
       return((int) '@');
+
     case MY_LEX_HOSTNAME:		// end '@' of user@hostname
       for (c=lip->yyGet() ;
            my_isalnum(cs,c) || c == '.' || c == '_' ||  c == '$';
            c= lip->yyGet()) ;
       yylval->lex_str=get_token(lip, 0, lip->yyLength());
       return(LEX_HOSTNAME);
+
     case MY_LEX_SYSTEM_VAR:
       yylval->lex_str.str=(char*) lip->get_ptr();
       yylval->lex_str.length=1;
@@ -1250,6 +1261,7 @@
 			MY_LEX_OPERATOR_OR_IDENT :
 			MY_LEX_IDENT_OR_KEYWORD);
       return((int) '@');
+
     case MY_LEX_IDENT_OR_KEYWORD:
       /*
         We come here when we have found two '@' in a row.
@@ -1263,9 +1275,11 @@
 
       if (c == '.')
         lip->next_state=MY_LEX_IDENT_SEP;
+
       length= lip->yyLength();
       if (length == 0)
         return(ABORT_SYM);              // Names must be nonempty.
+
       if ((tokval= find_keyword(lip, length,0)))
       {
         lip->yyUnget();                         // Put back 'c'

=== modified file 'drizzled/sql_lex.h'
--- drizzled/sql_lex.h	2011-02-24 19:58:57 +0000
+++ drizzled/sql_lex.h	2011-02-25 21:25:11 +0000
@@ -24,10 +24,7 @@
   @defgroup Semantic_Analysis Semantic Analysis
 */
 #include <drizzled/message/table.pb.h>
-
-#include <drizzled/plugin/function.h>
 #include <drizzled/name_resolution_context.h>
-#include <drizzled/item/subselect.h>
 #include <drizzled/table_list.h>
 #include <drizzled/function/math/real.h>
 #include <drizzled/key_part_spec.h>
@@ -38,8 +35,9 @@
 #include <bitset>
 #include <string>
 
-namespace drizzled
-{
+namespace drizzled {
+
+namespace plugin { class Function; }
 
 class st_lex_symbol;
 class select_result_interceptor;
@@ -391,7 +389,6 @@
   inline bool is_union ();
 
   friend void lex_start(Session *session);
-  friend int subselect_union_engine::exec();
 
   List<Item> *get_unit_column_types();
 };

=== modified file 'drizzled/sql_parse.cc'
--- drizzled/sql_parse.cc	2011-02-24 19:58:57 +0000
+++ drizzled/sql_parse.cc	2011-02-25 21:25:11 +0000
@@ -53,6 +53,7 @@
 #include <drizzled/visibility.h>
 #include <drizzled/kill.h>
 #include <drizzled/schema.h>
+#include <drizzled/item/subselect.h>
 
 #include <limits.h>
 

=== modified file 'drizzled/sql_union.cc'
--- drizzled/sql_union.cc	2011-02-23 13:15:31 +0000
+++ drizzled/sql_union.cc	2011-02-24 19:13:33 +0000
@@ -27,6 +27,7 @@
 #include <drizzled/select_union.h>
 #include <drizzled/sql_lex.h>
 #include <drizzled/session.h>
+#include <drizzled/item/subselect.h>
 
 namespace drizzled
 {

=== modified file 'drizzled/sql_yacc.yy'
--- drizzled/sql_yacc.yy	2011-02-24 18:16:42 +0000
+++ drizzled/sql_yacc.yy	2011-02-27 02:20:23 +0000
@@ -41,6 +41,7 @@
 #include <drizzled/alter_column.h>
 #include <drizzled/alter_drop.h>
 #include <drizzled/alter_info.h>
+#include <drizzled/item/subselect.h>
 
 int yylex(union ParserType *yylval, drizzled::Session *session);
 
@@ -176,7 +177,7 @@
   Currently there are 70 shift/reduce conflicts.
   We should not introduce new conflicts any more.
 */
-%expect 75
+%expect 79
 
 /*
    Comments for TOKENS.
@@ -572,6 +573,7 @@
 %nonassoc IN_SYM
 %nonassoc IS NULL_SYM TRUE_SYM FALSE_SYM
 
+%nonassoc CONCAT
 %nonassoc '|'
 %nonassoc '&'
 %nonassoc SHIFT_LEFT SHIFT_RIGHT
@@ -619,7 +621,7 @@
         text_string opt_gconcat_separator
 
 %type <field_val>
-      field_definition
+      field_def
       int_type
       real_type
 
@@ -635,7 +637,6 @@
 
 %type <num>
         order_dir
-        field_def
         opt_table_options
         all_or_any opt_distinct
         union_option
@@ -662,7 +663,11 @@
         predicate bit_expr
         table_wild simple_expr udf_expr
         expr_or_default set_expr_or_default
-        signed_literal now_or_signed_literal opt_escape
+        opt_escape
+        signed_literal
+        integer_literal
+        date_literal
+        boolean_literal
         simple_ident_q
         field_or_var limit_option
         function_call_keyword
@@ -742,7 +747,11 @@
         opt_delete_option varchar
         opt_outer table_list table_name
         opt_option opt_place
-        opt_attribute opt_attribute_list attribute
+        opt_attribute
+        opt_attribute_boolean
+        opt_attribute_timestamp
+        opt_attribute_number
+        opt_attribute_string
         flush_options flush_option
         equal optional_braces
         normal_join
@@ -1006,7 +1015,7 @@
 
 opt_if_not_exists:
           /* empty */ { $$= false; }
-        | IF not EXISTS { $$= true; YYSession->getLex()->setExists(); }
+        | IF NOT_SYM EXISTS { $$= true; YYSession->getLex()->setExists(); }
         ;
 
 opt_create_table_options:
@@ -1171,7 +1180,7 @@
           {
             parser::buildCreateFieldIdent(Lex);
           }
-          field_def
+          field_def opt_attribute_comment
           {
             statement::CreateTable *statement= (statement::CreateTable *)Lex->statement;
 
@@ -1193,104 +1202,96 @@
         ;
 
 field_def:
-          field_definition opt_attribute {}
-        ;
-
-field_definition:
-          int_type ignored_field_number_length opt_field_number_signed opt_zerofill
-          { 
-            $$= parser::buildIntegerColumn(Lex, $1, ($3 or $4));
-          }
-        | real_type opt_precision
-          {
-            assert ($1 == DRIZZLE_TYPE_DOUBLE);
-            $$= parser::buildDoubleColumn(Lex);
-          }
-        | char '(' NUM ')'
-          {
-            $$= parser::buildVarcharColumn(Lex, $3.str);
-          }
-        | char
-          {
-            $$= parser::buildVarcharColumn(Lex, "1");
-          }
-        | varchar '(' NUM ')'
-          {
-            $$= parser::buildVarcharColumn(Lex, $3.str);
-          }
-        | VARBINARY '(' NUM ')'
-          {
-            $$= parser::buildVarbinaryColumn(Lex, $3.str);
-          }
-        | DATE_SYM
-          {
-            $$=DRIZZLE_TYPE_DATE;
-
-            if (Lex->field())
-              Lex->field()->set_type(message::Table::Field::DATE);
-          }
-        | TIME_SYM
-          {
-            $$=DRIZZLE_TYPE_TIME;
-
-            if (Lex->field())
-              Lex->field()->set_type(message::Table::Field::TIME);
-          }
-        | TIMESTAMP_SYM
+          TIMESTAMP_SYM opt_attribute_timestamp
           {
             $$=parser::buildTimestampColumn(Lex, NULL);
           }
-        | TIMESTAMP_SYM '(' NUM ')'
+        | TIMESTAMP_SYM '(' NUM ')' opt_attribute_timestamp
           {
             $$=parser::buildTimestampColumn(Lex, $3.str);
-          }
-        | DATETIME_SYM
+          } 
+        | DATETIME_SYM opt_attribute_timestamp
           {
             $$=DRIZZLE_TYPE_DATETIME;
 
             if (Lex->field())
               Lex->field()->set_type(message::Table::Field::DATETIME);
-          }
-        | BLOB_SYM
-          {
-            $$= parser::buildBlobColumn(Lex);
-          }
-        | TEXT_SYM
+          } 
+        | int_type opt_attribute_number
+          { 
+            $$= $1;
+          }
+        | CHAR_SYM opt_attribute_string
+          {
+            $$= parser::buildVarcharColumn(Lex, "1");
+          }
+        | varchar '(' NUM ')' opt_attribute_string
+          {
+            $$= parser::buildVarcharColumn(Lex, $3.str);
+          }
+        | TEXT_SYM opt_attribute_string
           {
             $$=DRIZZLE_TYPE_BLOB;
             Lex->length=(char*) 0; /* use default length */
 
-          if (Lex->field())
-            Lex->field()->set_type(message::Table::Field::BLOB);
-          }
-        | DECIMAL_SYM float_options
-          {
-            $$= parser::buildDecimalColumn(Lex);
-          }
-        | NUMERIC_SYM float_options
-          {
-            $$= parser::buildDecimalColumn(Lex);
-          }
-        | FIXED_SYM float_options
-          {
-            $$= parser::buildDecimalColumn(Lex);
+            if (Lex->field())
+              Lex->field()->set_type(message::Table::Field::BLOB);
           }
         | ENUM_SYM
           {
             Lex->interval_list.clear();
           }
-          '(' string_list ')'
+          '(' string_list ')' opt_attribute_string
           {
             $$=DRIZZLE_TYPE_ENUM;
 
             if (Lex->field())
               Lex->field()->set_type(message::Table::Field::ENUM);
           }
-        | UUID_SYM
+        | VARBINARY '(' NUM ')' opt_attribute
+          {
+            $$= parser::buildVarbinaryColumn(Lex, $3.str);
+          }
+        | real_type opt_attribute_number
+          {
+            assert ($1 == DRIZZLE_TYPE_DOUBLE);
+            $$= parser::buildDoubleColumn(Lex);
+          } 
+        | DATE_SYM opt_attribute
+          {
+            $$=DRIZZLE_TYPE_DATE;
+
+            if (Lex->field())
+              Lex->field()->set_type(message::Table::Field::DATE);
+          }
+        | TIME_SYM opt_attribute
+          {
+            $$=DRIZZLE_TYPE_TIME;
+
+            if (Lex->field())
+              Lex->field()->set_type(message::Table::Field::TIME);
+          }
+        | BLOB_SYM opt_attribute
+          {
+            $$= parser::buildBlobColumn(Lex);
+          }
+        | DECIMAL_SYM float_options opt_attribute_number
+          {
+            $$= parser::buildDecimalColumn(Lex);
+          }
+        | NUMERIC_SYM float_options opt_attribute_number
+          {
+            $$= parser::buildDecimalColumn(Lex);
+          }
+        | FIXED_SYM float_options opt_attribute_number
+          {
+            $$= parser::buildDecimalColumn(Lex);
+          }
+        | UUID_SYM opt_attribute
           {
             $$= parser::buildUuidColumn(Lex);
           }
-        | BOOLEAN_SYM
+        | BOOLEAN_SYM opt_attribute_boolean
           {
             $$= parser::buildBooleanColumn(Lex);
           }
@@ -1300,40 +1301,36 @@
           }
         ;
 
-char:
-          CHAR_SYM {}
-        ;
-
 varchar:
-          char VARYING {}
-        | VARCHAR_SYM {}
+          CHAR_SYM VARYING
+          { }
+        | CHAR_SYM
+          { }
+        | VARCHAR_SYM
+          { }
         ;
 
 int_type:
-          INT_SYM 
-          {
-            $$= DRIZZLE_TYPE_LONG;
-          }
-        | BOOL_SYM
-          {
-            $$= DRIZZLE_TYPE_LONG;
-          }
-        | BIGINT_SYM
-          {
-            $$= DRIZZLE_TYPE_LONGLONG;
+          INT_SYM  ignored_field_number_length opt_field_number_signed opt_zerofill 
+          {
+            $$= parser::buildIntegerColumn(Lex, DRIZZLE_TYPE_LONG, ($3 or $4));
+          }
+        | BIGINT_SYM ignored_field_number_length opt_field_number_signed opt_zerofill 
+          {
+            $$= parser::buildIntegerColumn(Lex, DRIZZLE_TYPE_LONGLONG, ($3 or $4));
           }
         ;
 
 real_type:
-          REAL
-          {
-            $$= DRIZZLE_TYPE_DOUBLE;
-          }
-        | DOUBLE_SYM
-          {
-            $$= DRIZZLE_TYPE_DOUBLE;
-          }
-        | DOUBLE_SYM PRECISION
+          REAL opt_precision
+          {
+            $$= DRIZZLE_TYPE_DOUBLE;
+          }
+        | DOUBLE_SYM opt_precision
+          {
+            $$= DRIZZLE_TYPE_DOUBLE;
+          }
+        | DOUBLE_SYM PRECISION opt_precision
           {
             $$= DRIZZLE_TYPE_DOUBLE;
           }
@@ -1386,22 +1383,12 @@
           {}
         ;
 
-opt_attribute:
-          /* empty */ {}
-        | opt_attribute_list {}
-        ;
-
-opt_attribute_list:
-          opt_attribute_list attribute {}
-        | attribute
-        ;
-
-attribute:
+opt_attribute_not_null:
           NULL_SYM
           {
             Lex->type&= ~ NOT_NULL_FLAG;
           }
-        | not NULL_SYM
+        | NOT_SYM NULL_SYM
           {
             Lex->type|= NOT_NULL_FLAG;
 
@@ -1410,26 +1397,10 @@
               Lex->field()->mutable_constraints()->set_is_notnull(true);
             }
           }
-        | DEFAULT now_or_signed_literal
-          {
-            statement::AlterTable *statement= (statement::AlterTable *)Lex->statement;
+        ;
 
-            statement->default_value=$2;
-            statement->alter_info.flags.set(ALTER_COLUMN_DEFAULT);
-          }
-        | ON UPDATE_SYM NOW_SYM optional_braces
-          {
-            ((statement::AlterTable *)Lex->statement)->on_update_value= new Item_func_now_local();
-          }
-        | AUTO_INC
-          {
-            parser::buildAutoOnColumn(Lex);
-          }
-        | SERIAL_SYM DEFAULT VALUE_SYM
-          {
-            (void)parser::buildSerialColumn(Lex);
-          }
-        | opt_primary KEY_SYM
+opt_attribute_index:
+          opt_primary KEY_SYM
           {
             parser::buildPrimaryOnColumn(Lex);
           }
@@ -1441,13 +1412,57 @@
           {
             parser::buildKeyOnColumn(Lex);
           }
-        | COMMENT_SYM TEXT_STRING_sys
-          {
-            statement::AlterTable *statement= (statement::AlterTable *)Lex->statement;
-            statement->comment= $2;
-
-            if (Lex->field())
-              Lex->field()->set_comment($2.str);
+        ;
+
+opt_attribute:
+          /* empty */ {}
+        | opt_attribute opt_attribute_not_null
+          { }
+        | opt_attribute DEFAULT signed_literal
+          {
+            statement::AlterTable *statement= (statement::AlterTable *)Lex->statement;
+
+            statement->default_value= $3;
+            statement->alter_info.flags.set(ALTER_COLUMN_DEFAULT);
+          }
+        | opt_attribute opt_attribute_index
+          { }
+        ;
+
+opt_attribute_boolean:
+          /* empty */ { }
+        | opt_attribute_boolean opt_attribute_not_null
+          { }
+        | opt_attribute_boolean DEFAULT boolean_literal
+          {
+            statement::AlterTable *statement= (statement::AlterTable *)Lex->statement;
+
+            statement->default_value= $3;
+            statement->alter_info.flags.set(ALTER_COLUMN_DEFAULT);
+          }
+        | opt_attribute_boolean opt_attribute_index
+          { }
+        ;
+
+opt_attribute_string:
+          /* empty */ {}
+        | opt_attribute_list_string {}
+        ;
+
+opt_attribute_list_string:
+          opt_attribute_list_string attribute_string {}
+        | attribute_string
+        ;
+
+attribute_string:
+          opt_attribute_not_null
+          { }
+        | DEFAULT signed_literal
+          {
+            statement::AlterTable *statement= (statement::AlterTable *)Lex->statement;
+
+            statement->default_value=$2;
+            statement->alter_info.flags.set(ALTER_COLUMN_DEFAULT);
           }
         | COLLATE_SYM collation_name
           {
@@ -1462,13 +1477,87 @@
               Lex->charset=$2;
             }
           }
-        ;
-
-now_or_signed_literal:
-          NOW_SYM optional_braces
-          { $$= new Item_func_now_local(); }
-        | signed_literal
-          { $$=$1; }
+        | opt_attribute_index
+          { }
+        ;
+
+opt_attribute_number:
+          /* empty */ {}
+        | opt_attribute_list_integer {}
+        ;
+
+opt_attribute_list_integer:
+          opt_attribute_list_integer attribute_integer {}
+        | attribute_integer
+        ;
+
+attribute_integer:
+          opt_attribute_not_null
+          { }
+        | AUTO_INC
+          {
+            parser::buildAutoOnColumn(Lex);
+          }
+        | SERIAL_SYM DEFAULT VALUE_SYM
+          {
+            (void)parser::buildSerialColumn(Lex);
+          }
+        | DEFAULT integer_literal
+          {
+            statement::AlterTable *statement= (statement::AlterTable *)Lex->statement;
+
+            statement->default_value=$2;
+            statement->alter_info.flags.set(ALTER_COLUMN_DEFAULT);
+          }
+        | opt_attribute_index
+          { }
+        ;
+
+opt_attribute_timestamp:
+          /* empty */ {}
+        | opt_attribute_list_timestamp {}
+        ;
+
+opt_attribute_list_timestamp:
+          opt_attribute_list_timestamp attribute_timestamp {}
+        | attribute_timestamp
+        ;
+
+attribute_timestamp:
+          opt_attribute_not_null
+          { }
+        | DEFAULT NOW_SYM optional_braces
+          {
+            statement::AlterTable *statement= (statement::AlterTable *)Lex->statement;
+
+            statement->default_value= new Item_func_now_local();
+            statement->alter_info.flags.set(ALTER_COLUMN_DEFAULT);
+          }
+        | DEFAULT date_literal
+          {
+            statement::AlterTable *statement= (statement::AlterTable *)Lex->statement;
+
+            statement->default_value=$2;
+            statement->alter_info.flags.set(ALTER_COLUMN_DEFAULT);
+          }
+        | ON UPDATE_SYM NOW_SYM optional_braces
+          {
+            ((statement::AlterTable *)Lex->statement)->on_update_value= new Item_func_now_local();
+          }
+        | opt_attribute_index
+          { }
+        ;
+
+opt_attribute_comment:
+          /* empty */ { }
+        | COMMENT_SYM TEXT_STRING_sys
+          {
+            statement::AlterTable *statement= (statement::AlterTable *)Lex->statement;
+            statement->comment= $2;
+
+            if (Lex->field())
+              Lex->field()->set_comment($2.str);
+          }
         ;
 
 collation_name:
@@ -1788,7 +1877,7 @@
 
             Lex->setField(NULL);
           }
-          field_def
+          field_def opt_attribute_comment
           {
             statement::AlterTable *statement= (statement::AlterTable *)Lex->statement;
 
@@ -2233,7 +2322,7 @@
 
 /* all possible expressions */
 expr:
-          expr or expr %prec OR_SYM
+          expr OR_SYM expr %prec OR_SYM
           {
             /*
               Design notes:
@@ -2287,7 +2376,7 @@
             /* XOR is a proprietary extension */
             $$ = new (YYSession->mem_root) Item_cond_xor($1, $3);
           }
-        | expr and expr %prec AND_SYM
+        | expr AND_SYM expr %prec AND_SYM
           {
             /* See comments in rule expr: expr or expr */
             Item_cond_and *item1;
@@ -2331,31 +2420,53 @@
         | NOT_SYM expr %prec NOT_SYM
           { $$= negate_expression(YYSession, $2); }
         | bool_pri IS TRUE_SYM %prec IS
-          { $$= new (YYSession->mem_root) Item_func_istrue($1); }
-        | bool_pri IS not TRUE_SYM %prec IS
-          { $$= new (YYSession->mem_root) Item_func_isnottrue($1); }
+          { 
+            $$= new (YYSession->mem_root) Item_func_istrue($1);
+          }
+        | bool_pri IS NOT_SYM TRUE_SYM %prec IS
+          {
+            $$= new (YYSession->mem_root) Item_func_isnottrue($1);
+          }
         | bool_pri IS FALSE_SYM %prec IS
-          { $$= new (YYSession->mem_root) Item_func_isfalse($1); }
-        | bool_pri IS not FALSE_SYM %prec IS
-          { $$= new (YYSession->mem_root) Item_func_isnotfalse($1); }
+          {
+            $$= new (YYSession->mem_root) Item_func_isfalse($1);
+          }
+        | bool_pri IS NOT_SYM FALSE_SYM %prec IS
+          {
+            $$= new (YYSession->mem_root) Item_func_isnotfalse($1);
+          }
         | bool_pri IS UNKNOWN_SYM %prec IS
-          { $$= new Item_func_isnull($1); }
-        | bool_pri IS not UNKNOWN_SYM %prec IS
-          { $$= new Item_func_isnotnull($1); }
+          {
+            $$= new Item_func_isnull($1);
+          }
+        | bool_pri IS NOT_SYM UNKNOWN_SYM %prec IS
+          {
+            $$= new Item_func_isnotnull($1);
+          }
         | bool_pri
         ;
 
 bool_pri:
           bool_pri IS NULL_SYM %prec IS
-          { $$= new Item_func_isnull($1); }
-        | bool_pri IS not NULL_SYM %prec IS
-          { $$= new Item_func_isnotnull($1); }
+          {
+            $$= new Item_func_isnull($1);
+          }
+        | bool_pri IS NOT_SYM NULL_SYM %prec IS
+          {
+            $$= new Item_func_isnotnull($1);
+          }
         | bool_pri EQUAL_SYM predicate %prec EQUAL_SYM
-          { $$= new Item_func_equal($1,$3); }
+          {
+            $$= new Item_func_equal($1,$3);
+          }
         | bool_pri comp_op predicate %prec '='
-          { $$= (*$2)(0)->create($1,$3); }
+          {
+            $$= (*$2)(0)->create($1,$3);
+          }
         | bool_pri comp_op all_or_any '(' subselect ')' %prec '='
-          { $$= all_any_subquery_creator($1, $2, $3, $5); }
+          {
+            $$= all_any_subquery_creator($1, $2, $3, $5);
+          }
         | predicate
         ;
 
@@ -2364,7 +2475,7 @@
           {
             $$= new (YYSession->mem_root) Item_in_subselect($1, $4);
           }
-        | bit_expr not IN_SYM '(' subselect ')'
+        | bit_expr NOT_SYM IN_SYM '(' subselect ')'
           {
             Item *item= new (YYSession->mem_root) Item_in_subselect($1, $5);
             $$= negate_expression(YYSession, item);
@@ -2379,11 +2490,11 @@
             $6->push_front($1);
             $$= new (YYSession->mem_root) Item_func_in(*$6);
           }
-        | bit_expr not IN_SYM '(' expr ')'
+        | bit_expr NOT_SYM IN_SYM '(' expr ')'
           {
             $$= parser::handle_sql2003_note184_exception(YYSession, $1, false, $5);
           }
-        | bit_expr not IN_SYM '(' expr ',' expr_list ')'
+        | bit_expr NOT_SYM IN_SYM '(' expr ',' expr_list ')'
           {
             $7->push_front($5);
             $7->push_front($1);
@@ -2395,7 +2506,7 @@
           {
             $$= new Item_func_between($1,$3,$5);
           }
-        | bit_expr not BETWEEN_SYM bit_expr AND_SYM predicate
+        | bit_expr NOT_SYM BETWEEN_SYM bit_expr AND_SYM predicate
           {
             Item_func_between *item= new Item_func_between($1,$4,$6);
             item->negate();
@@ -2405,7 +2516,7 @@
           { 
             $$= new Item_func_like($1,$3,$4,Lex->escape_used);
           }
-        | bit_expr not LIKE simple_expr opt_escape
+        | bit_expr NOT_SYM LIKE simple_expr opt_escape
           { 
             $$= new Item_func_not(new Item_func_like($1,$4,$5, Lex->escape_used));
           }
@@ -2419,7 +2530,7 @@
               DRIZZLE_YYABORT;
             }
           }
-        | bit_expr not REGEXP_SYM bit_expr
+        | bit_expr NOT_SYM REGEXP_SYM bit_expr
           { 
             List<Item> *args= new (YYSession->mem_root) List<Item>;
             args->push_back($1);
@@ -2451,23 +2562,41 @@
             $$= new function::bit::ShiftLeft($1, $3);
           }
         | bit_expr '+' bit_expr %prec '+'
-          { $$= new Item_func_plus($1,$3); }
+          {
+            $$= new Item_func_plus($1,$3);
+          }
         | bit_expr '-' bit_expr %prec '-'
-          { $$= new Item_func_minus($1,$3); }
+          { 
+            $$= new Item_func_minus($1,$3);
+          }
         | bit_expr '+' INTERVAL_SYM expr interval %prec '+'
-          { $$= new Item_date_add_interval($1,$4,$5,0); }
+          {
+            $$= new Item_date_add_interval($1,$4,$5,0);
+          }
         | bit_expr '-' INTERVAL_SYM expr interval %prec '-'
-          { $$= new Item_date_add_interval($1,$4,$5,1); }
+          {
+            $$= new Item_date_add_interval($1,$4,$5,1);
+          }
         | bit_expr '*' bit_expr %prec '*'
-          { $$= new Item_func_mul($1,$3); }
+          {
+            $$= new Item_func_mul($1,$3);
+          }
         | bit_expr '/' bit_expr %prec '/'
-          { $$= new Item_func_div(YYSession,$1,$3); }
+          {
+            $$= new Item_func_div(YYSession,$1,$3);
+          }
         | bit_expr '%' bit_expr %prec '%'
-          { $$= new Item_func_mod($1,$3); }
+          {
+            $$= new Item_func_mod($1,$3);
+          }
         | bit_expr DIV_SYM bit_expr %prec DIV_SYM
-          { $$= new Item_func_int_div($1,$3); }
+          {
+            $$= new Item_func_int_div($1,$3);
+          }
         | bit_expr MOD_SYM bit_expr %prec MOD_SYM
-          { $$= new Item_func_mod($1,$3); }
+          {
+            $$= new Item_func_mod($1,$3);
+          }
         | bit_expr '^' bit_expr
           {
             $$= new (YYSession->mem_root) function::bit::Xor($1, $3);
@@ -2475,18 +2604,6 @@
         | simple_expr
         ;
 
-or:
-          OR_SYM
-       ;
-
-and:
-          AND_SYM
-       ;
-
-not:
-          NOT_SYM
-        ;
-
 comp_op:
           '='     { $$ = &comp_eq_creator; }
         | GE     { $$ = &comp_ge_creator; }
@@ -2520,6 +2637,10 @@
           {
             Lex->setSumExprUsed();
           }
+        | simple_expr CONCAT simple_expr
+          {
+            $$= new (YYSession->mem_root) Item_func_concat(*YYSession, $1, $3);
+          }
         | '+' simple_expr %prec UMINUS { $$= $2; }
         | '-' simple_expr %prec UMINUS
           {
@@ -2892,7 +3013,7 @@
 function_call_generic:
           IDENT_sys '('
           {
-            const plugin::Function *udf= plugin::Function::get($1.str, $1.length);
+            const plugin::Function *udf= plugin::Function::get(std::string($1.str, $1.length));
 
             /* Temporary placing the result of getFunction in $3 */
             $<udf>$= udf;
@@ -4818,6 +4939,52 @@
         | TIMESTAMP_SYM text_literal { $$ = $2; }
         ;
 
+integer_literal:
+          text_literal { $$ = $1; }
+        | HEX_NUM { $$ = new Item_hex_string($1.str, $1.length);}
+        | BIN_NUM { $$= new Item_bin_string($1.str, $1.length); }
+        | NUM_literal { $$ = $1; }
+        | NULL_SYM
+          {
+            $$ = new Item_null();
+            YYSession->m_lip->next_state=MY_LEX_OPERATOR_OR_IDENT;
+          }
+        | '+' NUM_literal { $$ = $2; }
+        | '-' NUM_literal
+          {
+            $2->max_length++;
+            $$= $2->neg();
+          }
+        ;
+
+boolean_literal:
+          NULL_SYM
+          {
+            $$ = new Item_null();
+            YYSession->m_lip->next_state=MY_LEX_OPERATOR_OR_IDENT;
+          }
+        | FALSE_SYM
+          {
+            $$= new drizzled::item::False();
+          }
+        | TRUE_SYM
+          {
+            $$= new drizzled::item::True();
+          }
+        ;
+
+date_literal:
+          text_literal { $$ = $1; }
+        | NULL_SYM
+          {
+            $$ = new Item_null();
+            YYSession->m_lip->next_state=MY_LEX_OPERATOR_OR_IDENT;
+          }
+        | NUM_literal { $$ = $1; }
+        | DATE_SYM text_literal { $$ = $2; }
+        | TIMESTAMP_SYM text_literal { $$ = $2; }
+        ;
+
 NUM_literal:
           NUM
           {

=== modified file 'drizzled/statement/unlock_tables.cc'
--- drizzled/statement/unlock_tables.cc	2011-02-17 00:14:13 +0000
+++ drizzled/statement/unlock_tables.cc	2011-02-24 14:58:24 +0000
@@ -38,8 +38,12 @@
   if (getSession()->isGlobalReadLock())
   {
     getSession()->unlockGlobalReadLock();
-  }
-  getSession()->my_ok();
+    getSession()->my_ok();
+  }
+  else
+  {
+    my_error(ER_NO_LOCK_HELD, MYF(0));
+  }
 
   return false;
 }

=== modified file 'drizzled/symbol_hash.gperf'
--- drizzled/symbol_hash.gperf	2011-02-23 23:48:30 +0000
+++ drizzled/symbol_hash.gperf	2011-02-24 20:50:45 +0000
@@ -42,6 +42,7 @@
   unsigned int  tok;
 };
 %%
+||, CONCAT
 <=, LE
 <>, NE
 !=, NE

=== modified file 'drizzled/sys_var.h'
--- drizzled/sys_var.h	2011-02-15 20:03:56 +0000
+++ drizzled/sys_var.h	2011-02-23 20:25:56 +0000
@@ -76,7 +76,7 @@
 
 uint64_t fix_unsigned(Session *, uint64_t, const struct option *);
 
-const std::string &getServerHostname();
+DRIZZLED_API const std::string &getServerHostname();
 int sys_var_init();
 
 /**

=== modified file 'plugin/benchmark/benchmarkudf.cc'
--- plugin/benchmark/benchmarkudf.cc	2011-02-17 00:14:13 +0000
+++ plugin/benchmark/benchmarkudf.cc	2011-02-24 19:13:33 +0000
@@ -18,9 +18,9 @@
  */
 
 #include <config.h>
-
 #include <drizzled/error.h>
 #include <drizzled/internal/m_string.h>
+#include <drizzled/plugin/function.h>
 #include <drizzled/session.h>
 
 using namespace std;

=== modified file 'plugin/logging_query/logging_query.cc'
--- plugin/logging_query/logging_query.cc	2011-02-15 20:03:56 +0000
+++ plugin/logging_query/logging_query.cc	2011-02-23 20:26:53 +0000
@@ -242,6 +242,11 @@
       return false;
 
     Session::QueryString query_string(session->getQueryString());
+    if (query_string == NULL)
+    {
+      return false;
+    }
+
     if (re)
     {
       int this_pcre_rc;

=== modified file 'plugin/mysql_protocol/mysql_protocol.cc'
--- plugin/mysql_protocol/mysql_protocol.cc	2011-02-23 23:23:45 +0000
+++ plugin/mysql_protocol/mysql_protocol.cc	2011-02-25 21:25:54 +0000
@@ -32,9 +32,8 @@
 #include "mysql_protocol.h"
 #include "mysql_password.h"
 #include "options.h"
-
 #include <drizzled/identifier.h>
-
+#include <drizzled/plugin/function.h>
 #include <libdrizzle/constants.h>
 
 #define PROTOCOL_VERSION 10
@@ -525,47 +524,47 @@
       switch (field.type)
       {
       case DRIZZLE_TYPE_LONG:
-        pos[6]= 3;
+        pos[6]= DRIZZLE_COLUMN_TYPE_LONG;
         break;
 
       case DRIZZLE_TYPE_DOUBLE:
-        pos[6]= 5;
+        pos[6]= DRIZZLE_COLUMN_TYPE_DOUBLE;
         break;
 
       case DRIZZLE_TYPE_NULL:
-        pos[6]= 6;
+        pos[6]= DRIZZLE_COLUMN_TYPE_NULL;
         break;
 
       case DRIZZLE_TYPE_TIMESTAMP:
-        pos[6]= 7;
+        pos[6]= DRIZZLE_COLUMN_TYPE_TIMESTAMP;
         break;
 
       case DRIZZLE_TYPE_LONGLONG:
-        pos[6]= 8;
+        pos[6]= DRIZZLE_COLUMN_TYPE_LONGLONG;
         break;
 
       case DRIZZLE_TYPE_DATETIME:
-        pos[6]= 12;
+        pos[6]= DRIZZLE_COLUMN_TYPE_DATETIME;
         break;
 
       case DRIZZLE_TYPE_TIME:
-        pos[6]= 13;
+        pos[6]= DRIZZLE_COLUMN_TYPE_TIME;
         break;
 
       case DRIZZLE_TYPE_DATE:
-        pos[6]= 14;
+        pos[6]= DRIZZLE_COLUMN_TYPE_DATE;
         break;
 
       case DRIZZLE_TYPE_VARCHAR:
-        pos[6]= 15;
+        pos[6]= DRIZZLE_COLUMN_TYPE_VARCHAR;
         break;
 
       case DRIZZLE_TYPE_MICROTIME:
-        pos[6]= 15;
+        pos[6]= DRIZZLE_COLUMN_TYPE_VARCHAR;
         break;
 
       case DRIZZLE_TYPE_UUID:
-        pos[6]= 15;
+        pos[6]= DRIZZLE_COLUMN_TYPE_VARCHAR;
         break;
 
       case DRIZZLE_TYPE_BOOLEAN:
@@ -573,15 +572,15 @@
         break;
 
       case DRIZZLE_TYPE_DECIMAL:
-        pos[6]= (char)246;
+        pos[6]= (char)DRIZZLE_COLUMN_TYPE_NEWDECIMAL;
         break;
 
       case DRIZZLE_TYPE_ENUM:
-        pos[6]= (char)247;
+        pos[6]= (char)DRIZZLE_COLUMN_TYPE_ENUM;
         break;
 
       case DRIZZLE_TYPE_BLOB:
-        pos[6]= (char)252;
+        pos[6]= (char)DRIZZLE_COLUMN_TYPE_BLOB;
         break;
       }
     }

=== modified file 'plugin/shutdown_function/shutdown.cc'
--- plugin/shutdown_function/shutdown.cc	2011-02-17 00:14:13 +0000
+++ plugin/shutdown_function/shutdown.cc	2011-02-24 19:13:33 +0000
@@ -39,19 +39,18 @@
 #include <signal.h>
 #include <drizzled/session.h>
 #include <drizzled/function/str/strfunc.h>
+#include <drizzled/plugin/function.h>
 
 using namespace drizzled;
 
 #define SHUTDOWN_MESSAGE "Beginning shutdown"
 
-class Shutdown :public Item_str_func
+class Shutdown : public Item_str_func
 {
 public:
-  Shutdown() :
-    Item_str_func()
+  Shutdown()
   { }
 
-
   void fix_length_and_dec()
   {
     max_length= sizeof(SHUTDOWN_MESSAGE) * system_charset_info->mbmaxlen;

=== added directory 'plugin/slave'
=== added directory 'plugin/slave/docs'
=== added file 'plugin/slave/docs/index.rst'
--- plugin/slave/docs/index.rst	1970-01-01 00:00:00 +0000
+++ plugin/slave/docs/index.rst	2011-02-24 16:50:37 +0000
@@ -0,0 +1,89 @@
+Replication Slave
+=================
+
+Description
+-----------
+
+The replication slave plugin provides a native implementation of replication
+between two Drizzle processes.
+
+This plugin requires a master that is running with the InnoDB replication log
+enabled.
+
+Configuration
+-------------
+
+Most of the options that can be used to control the replication slave plugin
+can only be given in a configuration file. The only exception is the
+**config-file** option which designates the location of this configuration
+file.
+
+**slave.config-file**
+
+   Path to the replication slave configuration file.
+
+The options below are read from the configuration file.
+
+**master-host**
+
+   Hostname/IP address of the master server.
+
+**master-port**
+
+   Drizzle port used by the master server. Default is 3306.
+
+**master-user**
+
+   Username to use for connecting to the master server.
+
+**master-pass**
+
+   Password associated with the username given by **master-user**.
+
+**max-reconnects**
+
+   The number of reconnection attempts the slave plugin will try if the
+   master server becomes unreachable. Default is 10.
+
+**seconds-between-reconnects**
+
+   The number of seconds to wait between reconnect attempts when the master
+   server becomes unreachable. Default is 30.
+
+Implementation Details
+----------------------
+
+The replication slave plugin creates two worker threads, each accessing a
+work queue (implemented as an InnoDB table) that contains the replication
+events. This is a producer/consumer paradigm where one thread populates the
+queue (the producer), and the other thread (the consumer) reads events from
+the queue.
+
+The producer thread (or I/O thread) is in charge of connecting to the master
+server and pulling down replication events from the master's transaction
+log and storing them locally in the slave queue. It is required that the
+master use the InnoDB replication log (--innodb.replication-log=true).
+
+The consumer thread (or applier thread) reads the replication events from
+the local slave queue, applies them locally, and then deletes successfully
+applied events from the queue.
+
+Schemas and Tables
+------------------
+
+The slave plugin creates its own schema and set of tables to store its
+metadata. It stores everything in the **sys_replication** schema. The
+following are the tables that it will create:
+
+**sys_replication.io_state**
+
+   Stores metadata about the IO/producer thread.
+
+**sys_replication.applier_state**
+
+   Stores metadata about the applier/consumer thread.
+
+**sys_replication.queue**
+
+   The replication event queue.
+

=== added file 'plugin/slave/module.cc'
--- plugin/slave/module.cc	1970-01-01 00:00:00 +0000
+++ plugin/slave/module.cc	2011-02-17 14:20:50 +0000
@@ -0,0 +1,73 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2011 David Shrewsbury
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include <config.h>
+#include <plugin/slave/replication_slave.h>
+#include <drizzled/plugin.h>
+#include <drizzled/configmake.h>   // for SYSCONFDIR
+#include <drizzled/module/option_map.h>
+#include <boost/program_options.hpp>
+#include <boost/filesystem.hpp>
+#include <string>
+
+using namespace drizzled;
+using namespace std;
+
+namespace po= boost::program_options;
+namespace fs= boost::filesystem;
+
+namespace slave
+{
+
+static const fs::path DEFAULT_SLAVE_CFG_FILE= SYSCONFDIR "/slave.cfg";
+
+static string slave_config;
+
+static int init(module::Context &context)
+{
+  const module::option_map &vm= context.getOptions();
+
+  ReplicationSlave *slave= new ReplicationSlave(vm["config-file"].as<string>());
+  context.add(slave);
+  return 0;
+}
+
+static void init_options(drizzled::module::option_context &context)
+{
+  context("config-file",
+          po::value<string>()->default_value(DEFAULT_SLAVE_CFG_FILE.string()),
+          N_("Path to the slave configuration file"));
+}
+
+} /* namespace slave */
+
+DRIZZLE_DECLARE_PLUGIN
+{
+  DRIZZLE_VERSION_ID,
+  "slave",
+  "1.0",
+  "David Shrewsbury",
+  "Implements Drizzle replication slave.",
+  PLUGIN_LICENSE_GPL,
+  slave::init,
+  NULL,                  /* depends */
+  slave::init_options    /* config options */
+}
+DRIZZLE_DECLARE_PLUGIN_END;

=== added file 'plugin/slave/plugin.ini'
--- plugin/slave/plugin.ini	1970-01-01 00:00:00 +0000
+++ plugin/slave/plugin.ini	2011-02-15 18:05:37 +0000
@@ -0,0 +1,8 @@
+[plugin]
+version=1.0
+author=David Shrewsbury
+title=Replication Slave
+description=Provides slave replication services
+sources=module.cc replication_slave.cc queue_thread.cc queue_consumer.cc queue_producer.cc sql_executor.cc replication_schema.cc
+headers=replication_slave.h queue_thread.h queue_consumer.h queue_producer.h sql_executor.h replication_schema.h
+libs=${top_builddir}/libdrizzle/libdrizzle.la

=== added file 'plugin/slave/queue_consumer.cc'
--- plugin/slave/queue_consumer.cc	1970-01-01 00:00:00 +0000
+++ plugin/slave/queue_consumer.cc	2011-02-23 16:49:25 +0000
@@ -0,0 +1,374 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2011 David Shrewsbury
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include <config.h>
+#include <plugin/slave/queue_consumer.h>
+#include <drizzled/message/transaction.pb.h>
+#include <drizzled/message/statement_transform.h>
+#include <drizzled/sql/result_set.h>
+#include <drizzled/execute.h>
+#include <string>
+#include <vector>
+#include <boost/thread.hpp>
+#include <boost/lexical_cast.hpp>
+#include <google/protobuf/text_format.h>
+
+using namespace std;
+using namespace drizzled;
+
+namespace slave
+{
+
+bool QueueConsumer::init()
+{
+  setApplierState("", true);
+  return true;
+}
+
+
+void QueueConsumer::shutdown()
+{
+  setApplierState(getErrorMessage(), false);
+}
+
+
+bool QueueConsumer::process()
+{
+  TrxIdList completedTransactionIds;
+
+  getListOfCompletedTransactions(completedTransactionIds);
+
+  for (size_t x= 0; x < completedTransactionIds.size(); x++)
+  {
+    string commit_id;
+    uint64_t trx_id= completedTransactionIds[x];
+
+    vector<string> aggregate_sql;  /* final SQL to execute */
+    vector<string> segmented_sql;  /* carryover from segmented statements */
+
+    message::Transaction transaction;
+    uint32_t segment_id= 1;
+
+    while (getMessage(transaction, commit_id, trx_id, segment_id++))
+    {
+      convertToSQL(transaction, aggregate_sql, segmented_sql);
+      transaction.Clear();
+    }
+
+    /*
+     * The last message in a transaction should always have a commit_id
+     * value larger than 0, though other messages of the same transaction
+     * will have commit_id = 0.
+     */
+    assert((not commit_id.empty()) && (commit_id != "0"));
+    assert(segmented_sql.empty());
+
+    if (not aggregate_sql.empty())
+    {
+      /*
+       * Execution using drizzled::Execute requires some special escaping.
+       */
+      vector<string>::iterator agg_iter;
+      for (agg_iter= aggregate_sql.begin(); agg_iter != aggregate_sql.end(); ++agg_iter)
+      {
+        string &sql= *agg_iter;
+        string::iterator si= sql.begin();
+        for (; si != sql.end(); ++si)
+        {
+          if (*si == '\"')
+          {
+            si= sql.insert(si, '\\');
+            ++si;
+          }
+          else if (*si == '\\')
+          {
+            si= sql.insert(si, '\\');
+            ++si;
+            si= sql.insert(si, '\\');
+            ++si;
+            si= sql.insert(si, '\\');
+            ++si;
+          }
+        }
+      }
+    }
+
+    if (not executeSQLWithCommitId(aggregate_sql, commit_id))
+    {
+      return false;
+    }
+
+    if (not deleteFromQueue(trx_id))
+    {
+      return false;
+    }
+  }
+
+  return true;
+}
+
+
+bool QueueConsumer::getMessage(message::Transaction &transaction,
+                              string &commit_id,
+                              uint64_t trx_id,
+                              uint32_t segment_id)
+{
+  string sql("SELECT `msg`, `commit_order` FROM `sys_replication`.`queue`"
+             " WHERE `trx_id` = ");
+  sql.append(boost::lexical_cast<string>(trx_id));
+  sql.append(" AND `seg_id` = ", 16);
+  sql.append(boost::lexical_cast<string>(segment_id));
+
+  sql::ResultSet result_set(2);
+  Execute execute(*(_session.get()), true);
+  
+  execute.run(sql, result_set);
+  
+  assert(result_set.getMetaData().getColumnCount() == 2);
+
+  /* Really should only be 1 returned row */
+  uint32_t found_rows= 0;
+  while (result_set.next())
+  {
+    string msg= result_set.getString(0);
+    string com_id= result_set.getString(1);
+
+    if ((msg == "") || (found_rows == 1))
+      break;
+
+    /* Neither column should be NULL */
+    assert(result_set.isNull(0) == false);
+    assert(result_set.isNull(1) == false);
+
+    google::protobuf::TextFormat::ParseFromString(msg, &transaction);
+
+    commit_id= com_id;
+    found_rows++;
+  }
+
+  if (found_rows == 0)
+    return false;
+  
+  return true;
+}
+
+bool QueueConsumer::getListOfCompletedTransactions(TrxIdList &list)
+{
+  Execute execute(*(_session.get()), true);
+  
+  string sql("SELECT `trx_id` FROM `sys_replication`.`queue`"
+             " WHERE `commit_order` IS NOT NULL ORDER BY `commit_order` ASC");
+  
+  /* ResultSet size must match column count */
+  sql::ResultSet result_set(1);
+
+  execute.run(sql, result_set);
+
+  assert(result_set.getMetaData().getColumnCount() == 1);
+
+  while (result_set.next())
+  {
+    assert(result_set.isNull(0) == false);
+    string value= result_set.getString(0);
+    
+    /* empty string returned when no more results */
+    if (value != "")
+    {
+      list.push_back(boost::lexical_cast<uint64_t>(result_set.getString(0)));
+    }
+  }
+  
+  return true;
+}
+
+
+bool QueueConsumer::convertToSQL(const message::Transaction &transaction,
+                                vector<string> &aggregate_sql,
+                                vector<string> &segmented_sql)
+{
+  if (transaction.has_event())
+    return true;
+
+  size_t num_statements= transaction.statement_size();
+
+  /*
+   * Loop through all Statement messages within this Transaction and
+   * convert each to equivalent SQL statements. Complete Statements will
+   * be appended to aggregate_sql, while segmented Statements will remain
+   * in segmented_sql to be appended to until completed, or rolled back.
+   */
+
+  for (size_t idx= 0; idx < num_statements; idx++)
+  {
+    const message::Statement &statement= transaction.statement(idx);
+    
+    /* We won't bother with executing a rolled back transaction */
+    if (statement.type() == message::Statement::ROLLBACK)
+    {
+      assert(idx == (num_statements - 1));  /* should be the final Statement */
+      aggregate_sql.clear();
+      segmented_sql.clear();
+      break;
+    }
+
+    switch (statement.type())
+    {
+      /* DDL cannot be in a transaction, so precede with a COMMIT */
+      case message::Statement::TRUNCATE_TABLE:
+      case message::Statement::CREATE_SCHEMA:
+      case message::Statement::ALTER_SCHEMA:
+      case message::Statement::DROP_SCHEMA:
+      case message::Statement::CREATE_TABLE:
+      case message::Statement::ALTER_TABLE:
+      case message::Statement::DROP_TABLE:
+      {
+        segmented_sql.push_back("COMMIT");
+        break;
+      }
+
+      /* Cancel any ongoing statement */
+      case message::Statement::ROLLBACK_STATEMENT:
+      {
+        segmented_sql.clear();
+        continue;
+      }
+      
+      default:
+      {
+        break;
+      }
+    }
+
+    if (message::transformStatementToSql(statement, segmented_sql,
+                                         message::DRIZZLE, true))
+    {
+      return false;
+    }
+
+    if (isEndStatement(statement))
+    {
+      aggregate_sql.insert(aggregate_sql.end(),
+                           segmented_sql.begin(),
+                           segmented_sql.end());
+      segmented_sql.clear();
+    }
+  }
+
+  return true;
+}
+
+
+bool QueueConsumer::isEndStatement(const message::Statement &statement)
+{
+  switch (statement.type())
+  {
+    case (message::Statement::INSERT):
+    {
+      const message::InsertData &data= statement.insert_data();
+      if (not data.end_segment())
+        return false;
+      break;
+    }
+    case (message::Statement::UPDATE):
+    {
+      const message::UpdateData &data= statement.update_data();
+      if (not data.end_segment())
+        return false;
+      break;
+    }
+    case (message::Statement::DELETE):
+    {
+      const message::DeleteData &data= statement.delete_data();
+      if (not data.end_segment())
+        return false;
+      break;
+    }
+    default:
+      return true;
+  }
+  return true;
+}
+
+
+void QueueConsumer::setApplierState(const string &err_msg, bool status)
+{
+  vector<string> statements;
+  string sql;
+  string msg(err_msg);
+
+  if (not status)
+  {
+    sql= "UPDATE `sys_replication`.`applier_state` SET `status` = 'STOPPED'";
+  }
+  else
+  {
+    sql= "UPDATE `sys_replication`.`applier_state` SET `status` = 'RUNNING'";
+  }
+  
+  sql.append(", `error_msg` = '", 17);
+
+  /* Escape embedded quotes and statement terminators */
+  string::iterator it;
+  for (it= msg.begin(); it != msg.end(); ++it)
+  {
+    if (*it == '\'')
+    {
+      it= msg.insert(it, '\'');
+      ++it;  /* advance back to the quote */
+    }
+    else if (*it == ';')
+    {
+      it= msg.insert(it, '\\');
+      ++it;  /* advance back to the semicolon */
+    }
+  }
+  
+  sql.append(msg);
+  sql.append("'", 1);
+
+  statements.push_back(sql);
+  executeSQL(statements);
+}
+
+
+bool QueueConsumer::executeSQLWithCommitId(vector<string> &sql,
+                                           const string &commit_id)
+{
+  string tmp("UPDATE `sys_replication`.`applier_state`"
+             " SET `last_applied_commit_id` = ");
+  tmp.append(commit_id);
+  sql.push_back(tmp);
+  
+  return executeSQL(sql);
+}
+
+
+bool QueueConsumer::deleteFromQueue(uint64_t trx_id)
+{
+  string sql("DELETE FROM `sys_replication`.`queue` WHERE `trx_id` = ");
+  sql.append(boost::lexical_cast<std::string>(trx_id));
+
+  vector<string> sql_vect;
+  sql_vect.push_back(sql);
+
+  return executeSQL(sql_vect);
+}
+
+} /* namespace slave */

=== added file 'plugin/slave/queue_consumer.h'
--- plugin/slave/queue_consumer.h	1970-01-01 00:00:00 +0000
+++ plugin/slave/queue_consumer.h	2011-02-17 14:20:50 +0000
@@ -0,0 +1,130 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2011 David Shrewsbury
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_SLAVE_QUEUE_CONSUMER_H
+#define PLUGIN_SLAVE_QUEUE_CONSUMER_H
+
+#include <plugin/slave/queue_thread.h>
+#include <plugin/slave/sql_executor.h>
+#include <drizzled/session.h>
+
+namespace drizzled
+{
+  namespace message
+  {
+    class Transaction;
+  }
+}
+
+namespace slave
+{
+
+class QueueConsumer : public QueueThread, public SQLExecutor
+{
+public:
+  QueueConsumer() :
+    QueueThread(),
+    SQLExecutor("slave", "replication"),
+    _check_interval(5)
+  { }
+
+  bool init();
+  bool process();
+  void shutdown();
+
+  void setSleepInterval(uint32_t seconds)
+  {
+    _check_interval= seconds;
+  }
+
+  uint32_t getSleepInterval()
+  {
+    return _check_interval;
+  }
+  
+  /**
+   * Update applier status in state table.
+   *
+   * @param err_msg Error message string
+   * @param status false = STOPPED, true = RUNNING
+   */
+  void setApplierState(const std::string &err_msg, bool status);
+
+private:
+  typedef std::vector<uint64_t> TrxIdList;
+
+  /** Number of seconds to sleep between checking queue for messages */
+  uint32_t _check_interval;
+
+  bool getListOfCompletedTransactions(TrxIdList &list);
+
+  bool getMessage(drizzled::message::Transaction &transaction,
+                  std::string &commit_id,
+                  uint64_t trx_id,
+                  uint32_t segment_id);
+
+  /**
+   * Convert the given Transaction message into equivalent SQL.
+   *
+   * @param[in] transaction Transaction protobuf message to convert.
+   * @param[in,out] aggregate_sql Buffer for total SQL for this transaction.
+   * @param[in,out] segmented_sql Buffer for carried over segmented statements.
+   *
+   * @retval true Success
+   * @retval false Failure
+   */
+  bool convertToSQL(const drizzled::message::Transaction &transaction,
+                    std::vector<std::string> &aggregate_sql,
+                    std::vector<std::string> &segmented_sql);
+
+  /**
+   * Execute a batch of SQL statements.
+   *
+   * @param sql Batch of SQL statements to execute.
+   * @param commit_id Commit ID value to store in state table.
+   *
+   * @retval true Success
+   * @retval false Failure
+   */
+  bool executeSQLWithCommitId(std::vector<std::string> &sql,
+                              const std::string &commit_id);
+  
+  /**
+   * Remove messages for a given transaction from the queue.
+   *
+   * @param trx_id Transaction ID for the messages to remove.
+   *
+   * @retval true Success
+   * @retval false Failure
+   */
+  bool deleteFromQueue(uint64_t trx_id);
+
+  /**
+   * Determine if a Statement message is an end message.
+   *
+   * @retval true Is an end Statement message
+   * @retval false Is NOT an end Statement message
+   */
+  bool isEndStatement(const drizzled::message::Statement &statement);
+};
+
+} /* namespace slave */
+
+#endif /* PLUGIN_SLAVE_QUEUE_CONSUMER_H */

=== added file 'plugin/slave/queue_producer.cc'
--- plugin/slave/queue_producer.cc	1970-01-01 00:00:00 +0000
+++ plugin/slave/queue_producer.cc	2011-02-23 16:49:25 +0000
@@ -0,0 +1,475 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2011 David Shrewsbury
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include <config.h>
+#include <plugin/slave/queue_producer.h>
+#include <drizzled/errmsg_print.h>
+#include <drizzled/sql/result_set.h>
+#include <drizzled/execute.h>
+#include <drizzled/gettext.h>
+#include <drizzled/message/transaction.pb.h>
+#include <boost/lexical_cast.hpp>
+#include <google/protobuf/text_format.h>
+
+using namespace std;
+using namespace drizzled;
+
+namespace slave
+{
+
+QueueProducer::~QueueProducer()
+{
+  if (_is_connected)
+    closeConnection();
+}
+
+bool QueueProducer::init()
+{
+  setIOState("", true);
+  return openConnection();
+}
+
+bool QueueProducer::process()
+{
+  if (_saved_max_commit_id == 0)
+  {
+    if (not queryForMaxCommitId(&_saved_max_commit_id))
+    {
+      if (_last_return == DRIZZLE_RETURN_LOST_CONNECTION)
+      {
+        if (reconnect())
+        {
+          return true;    /* reconnect successful, try again */
+        }
+        else
+        {
+          _last_error_message= "Master offline";
+          return false;   /* reconnect failed, shutdown the thread */
+        }
+      }
+      else
+      {
+        return false;     /* unrecoverable error, shutdown the thread */
+      }
+    }
+  }
+
+  if (not queryForReplicationEvents(_saved_max_commit_id))
+  {
+    if (_last_return == DRIZZLE_RETURN_LOST_CONNECTION)
+    {
+      if (reconnect())
+      {
+        return true;    /* reconnect successful, try again */
+      }
+      else
+      {
+        _last_error_message= "Master offline";
+        return false;   /* reconnect failed, shutdown the thread */
+      }
+    }
+    else
+    {
+      return false;     /* unrecoverable error, shutdown the thread */
+    }
+  }
+
+  return true;
+}
+
+void QueueProducer::shutdown()
+{
+  setIOState(_last_error_message, false);
+  if (_is_connected)
+    closeConnection();
+}
+
+bool QueueProducer::reconnect()
+{
+  errmsg_printf(error::ERROR, _("Lost connection to master. Reconnecting."));
+
+  _is_connected= false;
+  _last_return= DRIZZLE_RETURN_OK;
+  _last_error_message.clear();
+  boost::posix_time::seconds duration(_seconds_between_reconnects);
+
+  uint32_t attempts= 1;
+
+  while (not openConnection())
+  {
+    if (attempts++ == _max_reconnects)
+      break;
+    boost::this_thread::sleep(duration);
+  }
+
+  return _is_connected;
+}
+
+bool QueueProducer::openConnection()
+{
+  if (drizzle_create(&_drizzle) == NULL)
+  {
+    _last_return= DRIZZLE_RETURN_INTERNAL_ERROR;
+    _last_error_message= "Replication slave: ";
+    _last_error_message.append(drizzle_error(&_drizzle));
+    errmsg_printf(error::ERROR, _("%s"), _last_error_message.c_str());
+    return false;
+  }
+  
+  if (drizzle_con_create(&_drizzle, &_connection) == NULL)
+  {
+    _last_return= DRIZZLE_RETURN_INTERNAL_ERROR;
+    _last_error_message= "Replication slave: ";
+    _last_error_message.append(drizzle_error(&_drizzle));
+    errmsg_printf(error::ERROR, _("%s"), _last_error_message.c_str());
+    return false;
+  }
+  
+  drizzle_con_set_tcp(&_connection, _master_host.c_str(), _master_port);
+  drizzle_con_set_auth(&_connection, _master_user.c_str(), _master_pass.c_str());
+
+  drizzle_return_t ret= drizzle_con_connect(&_connection);
+
+  if (ret != DRIZZLE_RETURN_OK)
+  {
+    _last_return= ret;
+    _last_error_message= "Replication slave: ";
+    _last_error_message.append(drizzle_error(&_drizzle));
+    errmsg_printf(error::ERROR, _("%s"), _last_error_message.c_str());
+    return false;
+  }
+  
+  _is_connected= true;
+
+  return true;
+}
+
+bool QueueProducer::closeConnection()
+{
+  drizzle_return_t ret;
+  drizzle_result_st result;
+
+  _is_connected= false;
+
+  if (drizzle_quit(&_connection, &result, &ret) == NULL)
+  {
+    _last_return= ret;
+    return false;
+  }
+
+  drizzle_result_free(&result);
+
+  return true;
+}
+
+bool QueueProducer::queryForMaxCommitId(uint64_t *max_commit_id)
+{
+  /*
+   * This SQL will get the maximum commit_id value we have pulled over from
+   * the master. We query two tables because either the queue will be empty,
+   * in which case the last_applied_commit_id will be the value we want, or
+   * we have yet to drain the queue,  we get the maximum value still in
+   * the queue.
+   */
+  string sql("SELECT MAX(x.cid) FROM"
+             " (SELECT MAX(`commit_order`) AS cid FROM `sys_replication`.`queue`"
+             "  UNION ALL SELECT `last_applied_commit_id` AS cid"
+             "  FROM `sys_replication`.`applier_state`) AS x");
+
+  sql::ResultSet result_set(1);
+  Execute execute(*(_session.get()), true);
+  execute.run(sql, result_set);
+  assert(result_set.getMetaData().getColumnCount() == 1);
+
+  /* Really should only be 1 returned row */
+  uint32_t found_rows= 0;
+  while (result_set.next())
+  {
+    string value= result_set.getString(0);
+
+    if ((value == "") || (found_rows == 1))
+      break;
+
+    assert(result_set.isNull(0) == false);
+    *max_commit_id= boost::lexical_cast<uint64_t>(value);
+    found_rows++;
+  }
+
+  if (found_rows == 0)
+  {
+    _last_error_message= "Could not determine last committed transaction.";
+    return false;
+  }
+
+  return true;
+}
+
+bool QueueProducer::queryForTrxIdList(uint64_t max_commit_id,
+                                      vector<uint64_t> &list)
+{
+  (void)list;
+  string sql("SELECT `id` FROM `data_dictionary`.`sys_replication_log`"
+             " WHERE `commit_id` > ");
+  sql.append(boost::lexical_cast<string>(max_commit_id));
+  sql.append(" ORDER BY `commit_id` LIMIT 25");
+
+  drizzle_return_t ret;
+  drizzle_result_st result;
+  drizzle_result_create(&_connection, &result);
+  drizzle_query_str(&_connection, &result, sql.c_str(), &ret);
+  
+  if (ret != DRIZZLE_RETURN_OK)
+  {
+    _last_return= ret;
+    _last_error_message= "Replication slave: ";
+    _last_error_message.append(drizzle_error(&_drizzle));
+    errmsg_printf(error::ERROR, _("%s"), _last_error_message.c_str());
+    return false;
+  }
+
+  ret= drizzle_result_buffer(&result);
+
+  if (ret != DRIZZLE_RETURN_OK)
+  {
+    _last_return= ret;
+    _last_error_message= "Replication slave: ";
+    _last_error_message.append(drizzle_error(&_drizzle));
+    errmsg_printf(error::ERROR, _("%s"), _last_error_message.c_str());
+    drizzle_result_free(&result);
+    return false;
+  }
+
+  drizzle_row_t row;
+
+  while ((row= drizzle_row_next(&result)) != NULL)
+  {
+    if (row[0])
+    {
+      list.push_back(boost::lexical_cast<uint32_t>(row[0]));
+    }
+    else
+    {
+      _last_return= ret;
+      _last_error_message= "Replication slave: Unexpected NULL for trx id";
+      errmsg_printf(error::ERROR, _("%s"), _last_error_message.c_str());
+      drizzle_result_free(&result);
+      return false;
+    }
+  }
+
+  drizzle_result_free(&result);
+  return true;
+}
+
+
+bool QueueProducer::queueInsert(const char *trx_id,
+                                const char *seg_id,
+                                const char *commit_id,
+                                const char *msg,
+                                const char *msg_length)
+{
+  message::Transaction message;
+
+  message.ParseFromArray(msg, boost::lexical_cast<int>(msg_length));
+
+  /*
+   * The SQL to insert our results into the local queue.
+   */
+  string sql= "INSERT INTO `sys_replication`.`queue`"
+              " (`trx_id`, `seg_id`, `commit_order`, `msg`) VALUES (";
+  sql.append(trx_id);
+  sql.append(", ", 2);
+  sql.append(seg_id);
+  sql.append(", ", 2);
+  sql.append(commit_id);
+  sql.append(", '", 3);
+
+  /*
+   * Ideally we would store the Transaction message in binary form, as it
+   * it stored on the master and tranferred to the slave. However, we are
+   * inserting using drizzle::Execute which doesn't really handle binary
+   * data. Until that is changed, we store as plain text.
+   */
+  string message_text;
+  google::protobuf::TextFormat::PrintToString(message, &message_text);  
+
+  /*
+   * Execution using drizzled::Execute requires some special escaping.
+   */
+  string::iterator it= message_text.begin();
+  for (; it != message_text.end(); ++it)
+  {
+    if (*it == '\"')
+    {
+      it= message_text.insert(it, '\\');
+      ++it;
+    }
+    else if (*it == '\'')
+    {
+      it= message_text.insert(it, '\\');
+      ++it;
+      it= message_text.insert(it, '\\');
+      ++it;
+    }
+    else if (*it == '\\')
+    {
+      it= message_text.insert(it, '\\');
+      ++it;
+      it= message_text.insert(it, '\\');
+      ++it;
+      it= message_text.insert(it, '\\');
+      ++it;
+    }
+  }
+
+  sql.append(message_text);
+  sql.append("')", 2);
+
+  vector<string> statements;
+  statements.push_back(sql);
+
+  if (not executeSQL(statements))
+  {
+    markInErrorState();
+    return false;
+  }
+
+  uint64_t tmp_commit_id= boost::lexical_cast<uint64_t>(commit_id);
+  if (tmp_commit_id > _saved_max_commit_id)
+    _saved_max_commit_id= tmp_commit_id;
+
+  return true;
+}
+
+
+bool QueueProducer::queryForReplicationEvents(uint64_t max_commit_id)
+{
+  vector<uint64_t> trx_id_list;
+
+  if (not queryForTrxIdList(max_commit_id, trx_id_list))
+    return false;
+
+  if (trx_id_list.size() == 0)    /* nothing to get from the master */
+  {
+    return true;
+  }
+
+  /*
+   * The SQL to pull everything we need from the master.
+   */
+  string sql= "SELECT `id`, `segid`, `commit_id`, `message`, `message_len` "
+              " FROM `data_dictionary`.`sys_replication_log` WHERE `id` IN (";
+
+  for (size_t x= 0; x < trx_id_list.size(); x++)
+  {
+    if (x > 0)
+      sql.append(", ", 2);
+    sql.append(boost::lexical_cast<string>(trx_id_list[x]));
+  }
+
+  sql.append(")", 1);
+
+  drizzle_return_t ret;
+  drizzle_result_st result;
+  drizzle_result_create(&_connection, &result);
+  drizzle_query_str(&_connection, &result, sql.c_str(), &ret);
+  
+  if (ret != DRIZZLE_RETURN_OK)
+  {
+    _last_return= ret;
+    _last_error_message= "Replication slave: ";
+    _last_error_message.append(drizzle_error(&_drizzle));
+    errmsg_printf(error::ERROR, _("%s"), _last_error_message.c_str());
+    return false;
+  }
+
+  /* TODO: Investigate 1-row-at-a-time buffering */
+
+  ret= drizzle_result_buffer(&result);
+
+  if (ret != DRIZZLE_RETURN_OK)
+  {
+    _last_return= ret;
+    _last_error_message= "Replication slave: ";
+    _last_error_message.append(drizzle_error(&_drizzle));
+    errmsg_printf(error::ERROR, _("%s"), _last_error_message.c_str());
+    drizzle_result_free(&result);
+    return false;
+  }
+
+  drizzle_row_t row;
+
+  while ((row= drizzle_row_next(&result)) != NULL)
+  {
+    if (not queueInsert(row[0], row[1], row[2], row[3], row[4]))
+    {
+      errmsg_printf(error::ERROR,
+                    _("Replication slave: Unable to insert into queue."));
+      return false;
+    }
+  }
+
+  drizzle_result_free(&result);
+
+  return true;
+}
+
+
+void QueueProducer::setIOState(const string &err_msg, bool status)
+{
+  vector<string> statements;
+  string sql;
+  string msg(err_msg);
+
+  if (not status)
+  {
+    sql= "UPDATE `sys_replication`.`io_state` SET `status` = 'STOPPED'";
+  }
+  else
+  {
+    sql= "UPDATE `sys_replication`.`io_state` SET `status` = 'RUNNING'";
+  }
+  
+  sql.append(", `error_msg` = '", 17);
+
+  /* Escape embedded quotes and statement terminators */
+  string::iterator it;
+  for (it= msg.begin(); it != msg.end(); ++it)
+  {
+    if (*it == '\'')
+    {
+      it= msg.insert(it, '\'');
+      ++it;  /* advance back to the quote */
+    }
+    else if (*it == ';')
+    {
+      it= msg.insert(it, '\\');
+      ++it;  /* advance back to the semicolon */
+    }
+  }
+  
+  sql.append(msg);
+  sql.append("'", 1);
+
+  statements.push_back(sql);
+  executeSQL(statements);
+}
+
+} /* namespace slave */

=== added file 'plugin/slave/queue_producer.h'
--- plugin/slave/queue_producer.h	1970-01-01 00:00:00 +0000
+++ plugin/slave/queue_producer.h	2011-02-23 16:09:03 +0000
@@ -0,0 +1,156 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2011 David Shrewsbury
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_SLAVE_QUEUE_PRODUCER_H
+#define PLUGIN_SLAVE_QUEUE_PRODUCER_H
+
+#include <plugin/slave/queue_thread.h>
+#include <plugin/slave/sql_executor.h>
+#include <client/client_priv.h>
+#include <string>
+#include <vector>
+
+namespace slave
+{
+  
+class QueueProducer : public QueueThread, public SQLExecutor
+{
+public:
+  QueueProducer() :
+    SQLExecutor("slave", "replication"),
+    _check_interval(5),
+    _master_port(3306),
+    _last_return(DRIZZLE_RETURN_OK),
+    _is_connected(false),
+    _saved_max_commit_id(0),
+    _max_reconnects(10),
+    _seconds_between_reconnects(30)
+  {}
+
+  virtual ~QueueProducer();
+
+  bool init();
+  bool process();
+  void shutdown();
+
+  void setSleepInterval(uint32_t seconds)
+  {
+    _check_interval= seconds;
+  }
+
+  uint32_t getSleepInterval()
+  {
+    return _check_interval;
+  }
+
+  void setMasterHost(const std::string &host)
+  {
+    _master_host= host;
+  }
+
+  void setMasterPort(uint16_t port)
+  {
+    _master_port= port;
+  }
+
+  void setMasterUser(const std::string &user)
+  {
+    _master_user= user;
+  }
+
+  void setMasterPassword(const std::string &password)
+  {
+    _master_pass= password;
+  }
+
+  void setMaxReconnectAttempts(uint32_t max)
+  {
+    _max_reconnects= max;
+  }
+
+  void setSecondsBetweenReconnects(uint32_t seconds)
+  {
+    _seconds_between_reconnects= seconds;
+  }
+
+private:
+  /** Number of seconds to sleep between checking queue for messages */
+  uint32_t _check_interval;
+
+  /* Master server connection parameters */
+  std::string _master_host;
+  uint16_t    _master_port;
+  std::string _master_user;
+  std::string _master_pass;
+
+  drizzle_st _drizzle;
+  drizzle_con_st _connection;
+  drizzle_return_t _last_return;
+
+  bool _is_connected;
+  uint64_t _saved_max_commit_id;
+  uint32_t _max_reconnects;
+  uint32_t _seconds_between_reconnects;
+
+  std::string _last_error_message;
+
+  /**
+   * Open connection to the master server.
+   */
+  bool openConnection();
+
+  /**
+   * Close connection to the master server.
+   */
+  bool closeConnection();
+
+  /**
+   * Attempt to reconnect to the master server.
+   *
+   * This method does not return until reconnect succeeds, or we exceed our
+   * maximum number of retries defined by _max_reconnects.
+   *
+   * @retval true Reconnect succeeded
+   * @retval false Reconnect failed
+   */
+  bool reconnect();
+
+  bool queryForMaxCommitId(uint64_t *max_commit_id);
+  bool queryForReplicationEvents(uint64_t max_commit_id);
+  bool queryForTrxIdList(uint64_t max_commit_id, std::vector<uint64_t> &list);
+  bool queueInsert(const char *trx_id,
+                   const char *seg_id,
+                   const char *commit_id,
+                   const char *msg,
+                   const char *msg_length);
+
+  /**
+   * Update IO thread status in state table.
+   *
+   * @param err_msg Error message string
+   * @param status false = STOPPED, true = RUNNING
+   */
+  void setIOState(const std::string &err_msg, bool status);
+
+};
+
+} /* namespace slave */
+
+#endif /* PLUGIN_SLAVE_QUEUE_PRODUCER_H */

=== added file 'plugin/slave/queue_thread.cc'
--- plugin/slave/queue_thread.cc	1970-01-01 00:00:00 +0000
+++ plugin/slave/queue_thread.cc	2011-02-17 14:20:50 +0000
@@ -0,0 +1,67 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2011 David Shrewsbury
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include <config.h>
+#include <plugin/slave/queue_thread.h>
+#include <drizzled/internal/my_pthread.h>
+#include <boost/thread.hpp>
+
+using namespace drizzled;
+
+namespace slave
+{
+
+void QueueThread::run(void)
+{
+  boost::posix_time::seconds duration(getSleepInterval());
+
+  /* thread setup needed to do things like create a Session */
+  internal::my_thread_init();
+  boost::this_thread::at_thread_exit(&internal::my_thread_end);
+
+  if (not init())
+    return;
+
+  while (1)
+  {
+    {
+      /* This uninterruptable block processes the message queue */
+      boost::this_thread::disable_interruption di;
+
+      if (not process())
+      {
+        shutdown();
+        return;
+      }
+    }
+
+    /* Interruptable only when not doing work (aka, sleeping) */
+    try
+    {
+      boost::this_thread::sleep(duration);
+    }
+    catch (boost::thread_interrupted &)
+    {
+      return;
+    }
+  }
+}
+
+} /* namespace slave */

=== added file 'plugin/slave/queue_thread.h'
--- plugin/slave/queue_thread.h	1970-01-01 00:00:00 +0000
+++ plugin/slave/queue_thread.h	2011-02-15 18:05:37 +0000
@@ -0,0 +1,79 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2011 David Shrewsbury
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_SLAVE_QUEUE_THREAD_H
+#define PLUGIN_SLAVE_QUEUE_THREAD_H
+
+#include <drizzled/internal/my_pthread.h>
+
+namespace slave
+{
+
+/**
+ * Interface for threads interacting with the replication queue.
+ *
+ * This class uses the Template Method design pattern to define the set of
+ * steps for a thread operating on the replication queue. An implementing
+ * class need only implement the init(), process(), and/or shutdown() methods.
+ * The implementing class need only pass run() method to thread initialization.
+ */
+
+class QueueThread
+{
+public:
+  virtual ~QueueThread()
+  {}
+  
+  void run(void);
+
+  /**
+   * Do any initialization work.
+   *
+   * @retval true Success
+   * @retval false Failure
+   */
+  virtual bool init()
+  {
+    return true;
+  }
+
+  /**
+   * Work to do at thread shutdown time.
+   */
+  virtual void shutdown()
+  {}
+
+  /**
+   * Method that actually does the work around the queue.
+   *
+   * Returning 'false' from this method currently causes the thread to
+   * shutdown.
+   *
+   * @retval true Success
+   * @retval false Failure
+   */
+  virtual bool process()= 0;
+
+  virtual uint32_t getSleepInterval()= 0;
+};
+
+} /* namespace slave */
+
+#endif /* PLUGIN_SLAVE_QUEUE_THREAD_H */

=== added file 'plugin/slave/replication_schema.cc'
--- plugin/slave/replication_schema.cc	1970-01-01 00:00:00 +0000
+++ plugin/slave/replication_schema.cc	2011-02-23 16:49:25 +0000
@@ -0,0 +1,133 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2011 David Shrewsbury
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include <config.h>
+#include <plugin/slave/replication_schema.h>
+#include <drizzled/execute.h>
+#include <drizzled/sql/result_set.h>
+#include <string>
+#include <vector>
+
+using namespace std;
+using namespace drizzled;
+
+namespace slave
+{
+
+bool ReplicationSchema::create()
+{
+  vector<string> sql;
+
+  sql.push_back("COMMIT");
+  sql.push_back("CREATE SCHEMA IF NOT EXISTS `sys_replication` REPLICATE=FALSE");
+
+  if (not executeSQL(sql))
+    return false;
+
+  /*
+   * Create our IO thread state information table if we need to.
+   */
+
+  sql.clear();
+  sql.push_back("COMMIT");
+  sql.push_back("CREATE TABLE IF NOT EXISTS `sys_replication`.`io_state` ("
+                " `status` VARCHAR(20) NOT NULL,"
+                " `error_msg` VARCHAR(250))"
+                " COMMENT = 'VERSION 1.0'");
+
+  if (not executeSQL(sql))
+    return false;
+
+  sql.clear();
+  sql.push_back("SELECT COUNT(*) FROM `sys_replication`.`io_state`");
+
+  {
+    sql::ResultSet result_set(1);
+    Execute execute(*(_session.get()), true);
+    execute.run(sql[0], result_set);
+    result_set.next();
+    string count= result_set.getString(0);
+
+    /* Must always be at least one row in the table */
+    if (count == "0")
+    {
+      sql.clear();
+      sql.push_back("INSERT INTO `sys_replication`.`io_state` (`status`)"
+                    " VALUES ('STOPPED')");
+      if (not executeSQL(sql))
+        return false;
+    }
+  }
+
+  /*
+   * Create our applier thread state information table if we need to.
+   */
+
+  sql.clear();
+  sql.push_back("COMMIT");
+  sql.push_back("CREATE TABLE IF NOT EXISTS `sys_replication`.`applier_state`"
+                " (`last_applied_commit_id` BIGINT NOT NULL PRIMARY KEY,"
+                " `status` VARCHAR(20) NOT NULL,"
+                " `error_msg` VARCHAR(250))"
+                " COMMENT = 'VERSION 1.0'");
+
+  if (not executeSQL(sql))
+    return false;
+
+  sql.clear();
+  sql.push_back("SELECT COUNT(*) FROM `sys_replication`.`applier_state`");
+
+  {
+    sql::ResultSet result_set(1);
+    Execute execute(*(_session.get()), true);
+    execute.run(sql[0], result_set);
+    result_set.next();
+    string count= result_set.getString(0);
+
+    /* Must always be at least one row in the table */
+    if (count == "0")
+    {
+      sql.clear();
+      sql.push_back("INSERT INTO `sys_replication`.`applier_state`"
+                    " (`last_applied_commit_id`, `status`)"
+                    " VALUES (0, 'STOPPED')");
+      if (not executeSQL(sql))
+        return false;
+    }
+  }
+
+  /*
+   * Create our message queue table if we need to.
+   */
+
+  sql.clear();
+  sql.push_back("COMMIT");
+  sql.push_back("CREATE TABLE IF NOT EXISTS `sys_replication`.`queue`"
+                " (`trx_id` BIGINT NOT NULL, `seg_id` INT NOT NULL,"
+                " `commit_order` BIGINT, `msg` BLOB,"
+                " PRIMARY KEY(`trx_id`, `seg_id`))"
+                " COMMENT = 'VERSION 1.0'");
+  if (not executeSQL(sql))
+    return false;
+
+  return true;
+}
+
+} /* namespace slave */

=== added file 'plugin/slave/replication_schema.h'
--- plugin/slave/replication_schema.h	1970-01-01 00:00:00 +0000
+++ plugin/slave/replication_schema.h	2011-02-17 14:20:50 +0000
@@ -0,0 +1,40 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2011 David Shrewsbury
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_SLAVE_REPLICATION_SCHEMA_H
+#define PLUGIN_SLAVE_REPLICATION_SCHEMA_H
+
+#include <plugin/slave/sql_executor.h>
+
+namespace slave
+{
+
+class ReplicationSchema : public SQLExecutor
+{
+public:
+  ReplicationSchema() : SQLExecutor("slave", "replication")
+  { }
+
+  bool create();
+};
+
+} /* namespace slave */
+
+#endif /* PLUGIN_SLAVE_REPLICATION_SCHEMA_H */

=== added file 'plugin/slave/replication_slave.cc'
--- plugin/slave/replication_slave.cc	1970-01-01 00:00:00 +0000
+++ plugin/slave/replication_slave.cc	2011-02-21 20:05:01 +0000
@@ -0,0 +1,100 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2011 David Shrewsbury
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include <config.h>
+#include <plugin/slave/replication_slave.h>
+#include <drizzled/program_options/config_file.h>
+#include <drizzled/errmsg_print.h>
+#include <boost/program_options.hpp>
+#include <fstream>
+
+using namespace std;
+using namespace drizzled;
+
+namespace po= boost::program_options;
+
+namespace slave
+{
+
+/* Gets called after all plugins are initialized. */
+void ReplicationSlave::startup(Session &session)
+{
+  (void)session;
+  if (not initWithConfig())
+  {
+    errmsg_printf(error::ERROR,
+                  _("Could not start slave services: %s\n"),
+                  getError().c_str());
+  }
+  else
+  {
+    _consumer_thread= boost::thread(&QueueConsumer::run, &_consumer);
+    _producer_thread= boost::thread(&QueueProducer::run, &_producer);
+  }
+}
+
+bool ReplicationSlave::initWithConfig()
+{
+  po::variables_map vm;
+  po::options_description slave_options("Options for the slave plugin");
+
+  slave_options.add_options()
+    ("master-host", po::value<string>()->default_value(""))
+    ("master-port", po::value<uint16_t>()->default_value(3306))
+    ("master-user", po::value<string>()->default_value(""))
+    ("master-pass", po::value<string>()->default_value(""))
+    ("max-reconnects", po::value<uint32_t>()->default_value(10))
+    ("seconds-between-reconnects", po::value<uint32_t>()->default_value(30));
+
+  ifstream cf_stream(_config_file.c_str());
+  po::store(drizzled::program_options::parse_config_file(cf_stream, slave_options), vm);
+
+  po::notify(vm);
+
+  if (vm.count("master-host"))
+    _producer.setMasterHost(vm["master-host"].as<string>());
+
+  if (vm.count("master-port"))
+    _producer.setMasterPort(vm["master-port"].as<uint16_t>());
+
+  if (vm.count("master-user"))
+    _producer.setMasterUser(vm["master-user"].as<string>());
+
+  if (vm.count("master-pass"))
+    _producer.setMasterPassword(vm["master-pass"].as<string>());
+
+  if (vm.count("max-reconnects"))
+    _producer.setMaxReconnectAttempts(vm["max-reconnects"].as<uint32_t>());
+
+  if (vm.count("seconds-between-reconnects"))
+    _producer.setSecondsBetweenReconnects(vm["seconds-between-reconnects"].as<uint32_t>());
+
+  /* setup schema and tables */
+  ReplicationSchema rs;
+  if (not rs.create())
+  {
+    _error= rs.getErrorMessage();
+    return false;
+  }
+
+  return true;
+}
+
+} /* namespace slave */

=== added file 'plugin/slave/replication_slave.h'
--- plugin/slave/replication_slave.h	1970-01-01 00:00:00 +0000
+++ plugin/slave/replication_slave.h	2011-02-17 14:20:50 +0000
@@ -0,0 +1,90 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2011 David Shrewsbury
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_SLAVE_REPLICATION_SLAVE_H
+#define PLUGIN_SLAVE_REPLICATION_SLAVE_H
+
+#include <plugin/slave/queue_consumer.h>
+#include <plugin/slave/queue_producer.h>
+#include <plugin/slave/replication_schema.h>
+#include <drizzled/plugin/daemon.h>
+#include <boost/thread.hpp>
+
+namespace drizzled
+{
+  class Session;
+}
+
+namespace slave
+{
+
+class ReplicationSlave : public drizzled::plugin::Daemon
+{
+public:
+
+  ReplicationSlave(const std::string &config)
+    : drizzled::plugin::Daemon("Replication Slave"),
+      _config_file(config)
+  {}
+  
+  ~ReplicationSlave()
+  {
+    _consumer_thread.interrupt();
+    _producer_thread.interrupt();
+  }
+
+  void startup(drizzled::Session &session);
+
+  /**
+   * Get the error message describing what went wrong during setup.
+   */
+  const std::string &getError() const
+  {
+    return _error;
+  }
+
+private:
+  std::string _config_file;
+  std::string _error;
+
+  QueueConsumer _consumer;
+  QueueProducer _producer;
+
+  /** Applier thread that will drain the work queue */
+  boost::thread _consumer_thread;
+
+  /** I/O thread that will populate the work queue */
+  boost::thread _producer_thread;
+
+  /**
+   * Initialize slave services with the given configuration file.
+   *
+   * In case of an error during initialization, you can call the getError()
+   * method to get a string describing what went wrong.
+   *
+   * @retval true Success
+   * @retval false Failure
+   */
+  bool initWithConfig();
+};
+  
+} /* namespace slave */
+
+#endif /* PLUGIN_SLAVE_REPLICATION_SLAVE_H */

=== added file 'plugin/slave/sql_executor.cc'
--- plugin/slave/sql_executor.cc	1970-01-01 00:00:00 +0000
+++ plugin/slave/sql_executor.cc	2011-02-17 14:20:50 +0000
@@ -0,0 +1,103 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2011 David Shrewsbury
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include <config.h>
+#include <plugin/slave/sql_executor.h>
+#include <drizzled/plugin/listen.h>
+#include <drizzled/plugin/client.h>
+#include <drizzled/catalog/local.h>
+#include <drizzled/execute.h>
+#include <drizzled/sql/result_set.h>
+#include <drizzled/errmsg_print.h>
+
+using namespace std;
+using namespace drizzled;
+
+namespace slave
+{
+
+SQLExecutor::SQLExecutor(const string &user, const string &schema)
+  : _in_error_state(false)
+{
+  /* setup a Session object */
+  _session= Session::make_shared(plugin::Listen::getNullClient(),
+                                 catalog::local());
+  identifier::User::shared_ptr user_id= identifier::User::make_shared();
+  user_id->setUser(user);
+  _session->setUser(user_id);
+  _session->set_db(schema);
+}
+
+
+bool SQLExecutor::executeSQL(vector<string> &sql)
+{
+  string combined_sql;
+
+  if (not _in_error_state)
+    _error_message.clear();
+
+  Execute execute(*(_session.get()), true);
+
+  vector<string>::iterator iter= sql.begin();
+
+  while (iter != sql.end())
+  {
+    combined_sql.append(*iter);
+    combined_sql.append("; ");
+    ++iter;
+  }
+
+  sql::ResultSet result_set(1);
+
+  /* Execute wraps the SQL to run within a transaction */
+  execute.run(combined_sql, result_set);
+
+  sql::Exception exception= result_set.getException();
+
+  drizzled::error_t err= exception.getErrorCode();
+
+  if ((err != drizzled::EE_OK) && (err != drizzled::ER_EMPTY_QUERY))
+  {
+    /* avoid recursive errors */
+    if (_in_error_state)
+      return true;
+
+    _in_error_state= true;
+    _error_message= "(SQLSTATE ";
+    _error_message.append(exception.getSQLState());
+    _error_message.append(") ");
+    _error_message.append(exception.getErrorMessage());
+
+    string bad_sql("Failure while executing:\n");
+    for (size_t y= 0; y < sql.size(); y++)
+    {
+      bad_sql.append(sql[y]);
+      bad_sql.append("\n");
+    }
+
+    errmsg_printf(error::ERROR, _("%s\n%s\n"),
+                  _error_message.c_str(), bad_sql.c_str());
+    return false;
+  }
+
+  return true;
+}
+
+} /* namespace slave */

=== added file 'plugin/slave/sql_executor.h'
--- plugin/slave/sql_executor.h	1970-01-01 00:00:00 +0000
+++ plugin/slave/sql_executor.h	2011-02-15 18:05:37 +0000
@@ -0,0 +1,73 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2011 David Shrewsbury
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_SLAVE_SQL_EXECUTOR_H
+#define PLUGIN_SLAVE_SQL_EXECUTOR_H
+
+#include <string>
+#include <vector>
+#include <drizzled/session.h>
+
+namespace slave
+{
+
+class SQLExecutor
+{
+public:
+
+  SQLExecutor(const std::string &user, const std::string &schema);
+
+  void markInErrorState()
+  {
+    _in_error_state= true;
+  }
+
+  void clearErrorState()
+  {
+    _in_error_state= false;
+  }
+
+  const std::string &getErrorMessage()
+  {
+    return _error_message;
+  }
+
+  /**
+   * Execute a batch of SQL statements.
+   *
+   * @param sql Batch of SQL statements to execute.
+   *
+   * @retval true Success
+   * @retval false Failure
+   */
+  bool executeSQL(std::vector<std::string> &sql);
+
+protected:
+  drizzled::Session::shared_ptr _session;
+
+private:
+  bool _in_error_state;
+  std::string _error_message;
+
+};
+
+} /* namespace slave */
+
+#endif /* PLUGIN_SLAVE_SQL_EXECUTOR_H */

=== modified file 'plugin/sleep/sleep.cc'
--- plugin/sleep/sleep.cc	2011-02-17 00:14:13 +0000
+++ plugin/sleep/sleep.cc	2011-02-24 19:13:33 +0000
@@ -31,6 +31,7 @@
 #include <drizzled/item/func.h>
 #include <drizzled/internal/my_pthread.h>
 #include <drizzled/function/str/strfunc.h>
+#include <drizzled/plugin/function.h>
 
 #include <string>
 

=== modified file 'plugin/transaction_log/module.cc'
--- plugin/transaction_log/module.cc	2011-02-21 19:16:54 +0000
+++ plugin/transaction_log/module.cc	2011-02-24 19:13:33 +0000
@@ -46,6 +46,7 @@
 #include <drizzled/gettext.h>
 #include <boost/program_options.hpp>
 #include <drizzled/module/option_map.h>
+#include <drizzled/plugin/function.h>
 
 namespace po= boost::program_options;
 using namespace std;

=== modified file 'po/POTFILES.in'
--- po/POTFILES.in	2011-02-24 00:55:22 +0000
+++ po/POTFILES.in	2011-02-24 16:51:37 +0000
@@ -104,4 +104,8 @@
 plugin/transaction_log/utilities/transaction_file_reader.cc
 plugin/transaction_log/utilities/transaction_reader.cc
 plugin/transaction_log/write_buffer.cc
+plugin/slave/module.cc
+plugin/slave/queue_producer.cc
+plugin/slave/replication_slave.cc
+plugin/slave/sql_executor.cc
 plugin/session_dictionary/sessions.cc

=== added file 'tests/r/concat.result'
--- tests/r/concat.result	1970-01-01 00:00:00 +0000
+++ tests/r/concat.result	2011-02-24 20:50:45 +0000
@@ -0,0 +1,9 @@
+SELECT "foo" || "dog";
+"foo" || "dog"
+foodog
+SELECT "1" || "2";
+"1" || "2"
+12
+SELECT 1 || 2;
+1 || 2
+12

=== modified file 'tests/r/flush.result'
--- tests/r/flush.result	2009-06-05 23:10:06 +0000
+++ tests/r/flush.result	2011-02-24 14:58:24 +0000
@@ -25,6 +25,7 @@
 flush tables with read lock;
 insert into t2 values(1);
 unlock tables;
+ERROR HY000: No lock is held by this connection.
 drop table t1, t2;
 select benchmark(200, (select sin(1))) > 1000;
 End of 5.0 tests

=== modified file 'tests/r/metadata.result'
--- tests/r/metadata.result	2011-01-06 18:00:47 +0000
+++ tests/r/metadata.result	2011-02-25 17:54:03 +0000
@@ -19,7 +19,7 @@
 def	test	t1	t1	f	f	2	3	0	Y	32768	2	63
 def	test	t1	t1	g	g	2	4	0	Y	32768	3	63
 def	test	t1	t1	h	h	9	7	0	Y	0	4	63
-def	test	t1	t1	j	j	7	10	0	Y	128	0	63
+def	test	t1	t1	j	j	15	10	0	Y	128	0	63
 def	test	t1	t1	k	k	4	26	0	Y	128	0	63
 def	test	t1	t1	l	l	6	19	0	Y	128	0	63
 def	test	t1	t1	m	m	10	4	0	Y	256	0	45

=== added file 'tests/randgen_tests/innodb_trx_log/multi_thread1.disabled'
--- tests/randgen_tests/innodb_trx_log/multi_thread1.disabled	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/innodb_trx_log/multi_thread1.disabled	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = single threaded test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/translog_drizzle.zz --grammar=conf/drizzle/translog_concurrent1.yy --Reporter=DrizzleInnoTrxLog  --queries=100 --threads=3
+
+[test_servers]
+servers = [[--transaction-log.enable]]

=== added file 'tests/randgen_tests/innodb_trx_log/multi_thread1_mix.disabled'
--- tests/randgen_tests/innodb_trx_log/multi_thread1_mix.disabled	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/innodb_trx_log/multi_thread1_mix.disabled	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = single threaded test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/translog_drizzle.zz --grammar=conf/drizzle/translog_concurrent1.yy --Reporter=DrizzleInnoTrxLog  --queries=50 --threads=3 --seed=time
+
+[test_servers]
+servers = [[--transaction-log.enable]]

=== added file 'tests/randgen_tests/innodb_trx_log/multi_thread2.disabled'
--- tests/randgen_tests/innodb_trx_log/multi_thread2.disabled	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/innodb_trx_log/multi_thread2.disabled	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = single threaded test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/translog_drizzle.zz --grammar=conf/drizzle/translog_concurrent2.yy --Reporter=DrizzleInnoTrxLog  --queries=75 --threads=3
+
+[test_servers]
+servers = [[--transaction-log.enable]]

=== added file 'tests/randgen_tests/innodb_trx_log/multi_thread2_mix.disabled'
--- tests/randgen_tests/innodb_trx_log/multi_thread2_mix.disabled	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/innodb_trx_log/multi_thread2_mix.disabled	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = single threaded test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/translog_drizzle.zz --grammar=conf/drizzle/translog_concurrent2.yy --Reporter=DrizzleInnoTrxLog  --queries=50 --threads=3 --seed=time
+
+[test_servers]
+servers = [[--transaction-log.enable]]

=== added file 'tests/randgen_tests/innodb_trx_log/multi_thread3.disabled'
--- tests/randgen_tests/innodb_trx_log/multi_thread3.disabled	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/innodb_trx_log/multi_thread3.disabled	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = single threaded test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/translog_drizzle.zz --grammar=conf/drizzle/translog_concurrent3.yy --Reporter=DrizzleInnoTrxLog  --queries=75 --threads=3
+
+[test_servers]
+servers = [[--transaction-log.enable]]

=== added file 'tests/randgen_tests/innodb_trx_log/multi_thread3_mix.disabled'
--- tests/randgen_tests/innodb_trx_log/multi_thread3_mix.disabled	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/innodb_trx_log/multi_thread3_mix.disabled	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = single threaded test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/translog_drizzle.zz --grammar=conf/drizzle/translog_concurrent3.yy --Reporter=DrizzleInnoTrxLog  --queries=50 --threads=3 --seed=time
+
+[test_servers]
+servers = [[--transaction-log.enable]]

=== added file 'tests/randgen_tests/innodb_trx_log/single_thread3.disabled'
--- tests/randgen_tests/innodb_trx_log/single_thread3.disabled	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/innodb_trx_log/single_thread3.disabled	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = single threaded test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/translog_drizzle.zz --grammar=conf/drizzle/translog_concurrent3.yy --Reporter=DrizzleInnoTrxLog  --queries=100 --threads=1
+
+[test_servers]
+servers = [[--transaction-log.enable]]

=== added file 'tests/randgen_tests/innodb_trx_log/trx_ordering.disabled'
--- tests/randgen_tests/innodb_trx_log/trx_ordering.disabled	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/innodb_trx_log/trx_ordering.disabled	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = simple trx ordering test
+
+[test_command]
+command = ./gentest.pl ---gendata=conf/drizzle/translog_ordering.zz --grammar=conf/drizzle/translog_ordering.yy --Reporter=DrizzleInnoTrxLog --threads=5  --queries=75
+
+[test_servers]
+servers = [[--transaction-log.enable]]

=== added file 'tests/randgen_tests/main/basic.cnf'
--- tests/randgen_tests/main/basic.cnf	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/main/basic.cnf	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = basic randgen test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/drizzle.zz --grammar=conf/drizzle/drizzle.yy --queries=1000 --threads=5
+
+[test_servers]
+servers = [[]]

=== added file 'tests/randgen_tests/main/blob.cnf'
--- tests/randgen_tests/main/blob.cnf	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/main/blob.cnf	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = blob test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/drizzle.zz --grammar=conf/engines/blobs.yy --Validators=None --queries=1000 --threads=3
+
+[test_servers]
+servers = [[]]

=== added file 'tests/randgen_tests/main/collations.cnf'
--- tests/randgen_tests/main/collations.cnf	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/main/collations.cnf	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = collations test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/drizzle.zz --grammar=conf/drizzle/collations_drizzle.yy --queries=1000 --threads=3
+
+[test_servers]
+servers = [[]]

=== added file 'tests/randgen_tests/main/combinations.cnf'
--- tests/randgen_tests/main/combinations.cnf	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/main/combinations.cnf	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = collations test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/combinations_drizzle.zz --grammar=conf/transactions/combinations.yy --queries=1000 --threads=3
+
+[test_servers]
+servers = [[]]

=== added file 'tests/randgen_tests/main/create_drop.cnf'
--- tests/randgen_tests/main/create_drop.cnf	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/main/create_drop.cnf	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = collations test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/drizzle.zz --grammar=conf/runtime/create_drop.yy --queries=1000 --threads=3
+
+[test_servers]
+servers = [[]]

=== added file 'tests/randgen_tests/main/database_consistency_after_recovery.disabled'
--- tests/randgen_tests/main/database_consistency_after_recovery.disabled	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/main/database_consistency_after_recovery.disabled	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = database recovery test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/transactions/transactions.zz --grammar=conf/transactions/transactions.yy --reporters=DrizzleRecoveryConsistency --queries=1000 --threads=3
+
+[test_servers]
+servers = [[]]

=== added file 'tests/randgen_tests/main/database_integrity_after_recovery.disabled'
--- tests/randgen_tests/main/database_integrity_after_recovery.disabled	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/main/database_integrity_after_recovery.disabled	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = database recovery test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/drizzle.zz --grammar=conf/drizzle/drizzle.yy --reporters=DrizzleRecovery --queries=1000 --threads=3
+
+[test_servers]
+servers = [[]]

=== added file 'tests/randgen_tests/main/drizzledump_restore.cnf'
--- tests/randgen_tests/main/drizzledump_restore.cnf	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/main/drizzledump_restore.cnf	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = repeatable read test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/drizzle.zz --grammar=conf/drizzle/drizzledump.yy --Validator=Drizzledump --queries=50 --threads=1
+
+[test_servers]
+servers = [[]]

=== added file 'tests/randgen_tests/main/drizzledump_restore_rand.cnf'
--- tests/randgen_tests/main/drizzledump_restore_rand.cnf	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/main/drizzledump_restore_rand.cnf	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = repeatable read test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/drizzle.zz --grammar=conf/drizzle/drizzledump.yy --Validator=Drizzledump --queries=25 --seed=time --threads=1
+
+[test_servers]
+servers = [[]]

=== added file 'tests/randgen_tests/main/limit_compare.cnf'
--- tests/randgen_tests/main/limit_compare.cnf	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/main/limit_compare.cnf	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = database recovery test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/drizzle.zz --grammar=conf/drizzle/limit_compare_drizzle.yy --queries=1000 --threads=3
+
+[test_servers]
+servers = [[]]

=== added file 'tests/randgen_tests/main/many_indexes.cnf'
--- tests/randgen_tests/main/many_indexes.cnf	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/main/many_indexes.cnf	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = database recovery test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/many_indexes_drizzle.zz --grammar=conf/drizzle/many_indexes_drizzle.yy --threads=1 --queries=10 --Validators=None
+
+[test_servers]
+servers = [[]]

=== modified file 'tests/randgen_tests/main/optimizer_subquery.cnf'
--- tests/randgen_tests/main/optimizer_subquery.cnf	2011-02-23 17:17:25 +0000
+++ tests/randgen_tests/main/optimizer_subquery.cnf	2011-02-24 22:01:43 +0000
@@ -2,7 +2,7 @@
 comment = optimizer subquery test 
 
 [test_command]
-command = ./gentest.pl --gendata=conf/drizzle/drizzle.zz --grammar=conf/drizzle/optimizer_subquery_drizzle.yy --queries=500 --threads=5
+command = ./gentest.pl --gendata=conf/drizzle/drizzle.zz --grammar=conf/drizzle/optimizer_subquery_drizzle.yy --queries=250 --threads=3
 
 [test_servers]
 servers = [[]]

=== added file 'tests/randgen_tests/main/orderby_validator.disabled'
--- tests/randgen_tests/main/orderby_validator.disabled	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/main/orderby_validator.disabled	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = repeatable read test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/drizzle.zz --grammar=conf/drizzle/subquery_semijoin_drizzle.yy --validators=OrderBy --queries=100 --threads=3
+
+[test_servers]
+servers = [[]]

=== added file 'tests/randgen_tests/main/outer_join_portable.cnf'
--- tests/randgen_tests/main/outer_join_portable.cnf	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/main/outer_join_portable.cnf	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = repeatable read test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/outer_join_drizzle.zz --grammar=conf/drizzle/outer_join_portable_drizzle.yy --queries=1000 --threads=3
+
+[test_servers]
+servers = [[]]

=== added file 'tests/randgen_tests/main/repeatable_read.cnf'
--- tests/randgen_tests/main/repeatable_read.cnf	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/main/repeatable_read.cnf	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = repeatable read test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/transactions/transactions.zz --grammar=conf/transactions/repeatable_read.yy --validators=RepeatableRead --queries=1000 --threads=3
+
+[test_servers]
+servers = [[]]

=== added file 'tests/randgen_tests/main/select_stability_validator.cnf'
--- tests/randgen_tests/main/select_stability_validator.cnf	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/main/select_stability_validator.cnf	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = repeatable read test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/drizzle.zz --grammar=conf/drizzle/subquery_materialization_drizzle.yy --validators=SelectStability --queries=250 --threads=3
+
+[test_servers]
+servers = [[]]

=== removed file 'tests/randgen_tests/main/simple.cnf'
--- tests/randgen_tests/main/simple.cnf	2011-02-23 17:17:25 +0000
+++ tests/randgen_tests/main/simple.cnf	1970-01-01 00:00:00 +0000
@@ -1,9 +0,0 @@
-[test_info]
-name = randgen_simple
-comment = basic, basic randgen test
-
-[test_command]
-command = ./gentest.pl --gendata=conf/drizzle/drizzle.zz --grammar=conf/drizzle/optimizer_subquery_drizzle.yy --queries=10 --threads=1
-
-[test_servers]
-servers = [[--transaction-log.enable]]

=== added file 'tests/randgen_tests/main/subquery.cnf'
--- tests/randgen_tests/main/subquery.cnf	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/main/subquery.cnf	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = repeatable read test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/drizzle.zz --grammar=conf/drizzle/subquery_drizzle.yy --queries=1000 --threads=3
+
+[test_servers]
+servers = [[]]

=== added file 'tests/randgen_tests/main/subquery_semijoin.cnf'
--- tests/randgen_tests/main/subquery_semijoin.cnf	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/main/subquery_semijoin.cnf	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = repeatable read test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/drizzle.zz --grammar=conf/drizzle/subquery_semijoin_drizzle.yy --queries=1000 --threads=3
+
+[test_servers]
+servers = [[]]

=== added file 'tests/randgen_tests/main/subquery_semijoin_nested.cnf'
--- tests/randgen_tests/main/subquery_semijoin_nested.cnf	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/main/subquery_semijoin_nested.cnf	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = repeatable read test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/drizzle.zz --grammar=conf/drizzle/subquery_semijoin_nested_drizzle.yy --queries=1000 --threads=3
+
+[test_servers]
+servers = [[]]

=== added file 'tests/randgen_tests/main/varchar.cnf'
--- tests/randgen_tests/main/varchar.cnf	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/main/varchar.cnf	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = repeatable read test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/varchar_drizzle.zz --grammar=conf/engines/varchar.yy --queries=1000 --threads=3
+
+[test_servers]
+servers = [[]]

=== added file 'tests/randgen_tests/slave_plugin/slave_plugin.cnf'
--- tests/randgen_tests/slave_plugin/slave_plugin.cnf	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/slave_plugin/slave_plugin.cnf	2011-02-24 22:01:43 +0000
@@ -0,0 +1,9 @@
+[test_info]
+name = randgen_slave_plugin
+comment = does NOT actually test the master-slave replication yet, but it will.
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/drizzle.zz --grammar=conf/drizzle/optimizer_subquery_drizzle.yy --queries=10 --threads=1
+
+[test_servers]
+servers = [[--innodb.replication-log=true],[--plugin-add=slave --slave.config-file=$MASTER_SERVER_SLAVE_CONFIG]]

=== removed file 'tests/randgen_tests/slave_plugin/slave_plugin.disabled'
--- tests/randgen_tests/slave_plugin/slave_plugin.disabled	2011-02-23 17:17:25 +0000
+++ tests/randgen_tests/slave_plugin/slave_plugin.disabled	1970-01-01 00:00:00 +0000
@@ -1,9 +0,0 @@
-[test_info]
-name = randgen_slave_plugin
-comment = basic, basic randgen test
-
-[test_command]
-command = ./gentest.pl --gendata=conf/drizzle/drizzle.zz --grammar=conf/drizzle/optimizer_subquery_drizzle.yy --queries=10 --threads=1
-
-[test_servers]
-servers = [[--innodb.replication-log=true],[--plugin-add=slave --slave.config-file=$MASTER_SERVER_SLAVE_CONFIG]]

=== added file 'tests/randgen_tests/trx_log/multi_thread1.disabled'
--- tests/randgen_tests/trx_log/multi_thread1.disabled	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/trx_log/multi_thread1.disabled	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = single threaded test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/translog_drizzle.zz --grammar=conf/drizzle/translog_concurrent1.yy --Reporter=DrizzleTransactionLog  --queries=100 --threads=3
+
+[test_servers]
+servers = [[--transaction-log.enable]]

=== added file 'tests/randgen_tests/trx_log/multi_thread1_mix.disabled'
--- tests/randgen_tests/trx_log/multi_thread1_mix.disabled	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/trx_log/multi_thread1_mix.disabled	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = single threaded test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/translog_drizzle.zz --grammar=conf/drizzle/translog_concurrent1.yy --Reporter=DrizzleTransactionLog  --queries=50 --threads=3 --seed=time
+
+[test_servers]
+servers = [[--transaction-log.enable]]

=== added file 'tests/randgen_tests/trx_log/multi_thread2.disabled'
--- tests/randgen_tests/trx_log/multi_thread2.disabled	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/trx_log/multi_thread2.disabled	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = single threaded test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/translog_drizzle.zz --grammar=conf/drizzle/translog_concurrent2.yy --Reporter=DrizzleTransactionLog  --queries=75 --threads=3
+
+[test_servers]
+servers = [[--transaction-log.enable]]

=== added file 'tests/randgen_tests/trx_log/multi_thread2_mix.disabled'
--- tests/randgen_tests/trx_log/multi_thread2_mix.disabled	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/trx_log/multi_thread2_mix.disabled	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = single threaded test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/translog_drizzle.zz --grammar=conf/drizzle/translog_concurrent2.yy --Reporter=DrizzleTransactionLog  --queries=50 --threads=3 --seed=time
+
+[test_servers]
+servers = [[--transaction-log.enable]]

=== added file 'tests/randgen_tests/trx_log/multi_thread3.disabled'
--- tests/randgen_tests/trx_log/multi_thread3.disabled	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/trx_log/multi_thread3.disabled	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = single threaded test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/translog_drizzle.zz --grammar=conf/drizzle/translog_concurrent3.yy --Reporter=DrizzleTransactionLog  --queries=75 --threads=3
+
+[test_servers]
+servers = [[--transaction-log.enable]]

=== added file 'tests/randgen_tests/trx_log/multi_thread3_mix.disabled'
--- tests/randgen_tests/trx_log/multi_thread3_mix.disabled	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/trx_log/multi_thread3_mix.disabled	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = single threaded test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/translog_drizzle.zz --grammar=conf/drizzle/translog_concurrent3.yy --Reporter=DrizzleTransactionLog  --queries=50 --threads=3 --seed=time
+
+[test_servers]
+servers = [[--transaction-log.enable]]

=== removed file 'tests/randgen_tests/trx_log/simple.cnf'
--- tests/randgen_tests/trx_log/simple.cnf	2011-02-23 17:17:25 +0000
+++ tests/randgen_tests/trx_log/simple.cnf	1970-01-01 00:00:00 +0000
@@ -1,9 +0,0 @@
-[test_info]
-name = randgen_simple
-comment = basic, basic randgen test
-
-[test_command]
-command = ./gentest.pl --gendata=conf/drizzle/drizzle.zz --grammar=conf/drizzle/optimizer_subquery_drizzle.yy --queries=10 --threads=1
-
-[test_servers]
-servers = [[--transaction-log.enable]]

=== added file 'tests/randgen_tests/trx_log/single_thread3.disabled'
--- tests/randgen_tests/trx_log/single_thread3.disabled	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/trx_log/single_thread3.disabled	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = single threaded test
+
+[test_command]
+command = ./gentest.pl --gendata=conf/drizzle/translog_drizzle.zz --grammar=conf/drizzle/translog_concurrent3.yy --Reporter=DrizzleTransactionLog  --queries=100 --threads=1
+
+[test_servers]
+servers = [[--transaction-log.enable]]

=== added file 'tests/randgen_tests/trx_log/trx_ordering.disabled'
--- tests/randgen_tests/trx_log/trx_ordering.disabled	1970-01-01 00:00:00 +0000
+++ tests/randgen_tests/trx_log/trx_ordering.disabled	2011-02-24 22:01:43 +0000
@@ -0,0 +1,8 @@
+[test_info]
+comment = simple trx ordering test
+
+[test_command]
+command = ./gentest.pl ---gendata=conf/drizzle/translog_ordering.zz --grammar=conf/drizzle/translog_ordering.yy --Reporter=DrizzleTransactionLog --threads=5  --queries=75
+
+[test_servers]
+servers = [[--transaction-log.enable]]

=== added file 'tests/suite/bool_type/r/default.result'
--- tests/suite/bool_type/r/default.result	1970-01-01 00:00:00 +0000
+++ tests/suite/bool_type/r/default.result	2011-02-26 03:41:37 +0000
@@ -0,0 +1,41 @@
+CREATE TABLE t1 (A BOOLEAN DEFAULT TRUE);
+SHOW CREATE TABLE t1;
+Table	Create Table
+t1	CREATE TABLE `t1` (
+  `A` BOOLEAN DEFAULT 'TRUE'
+) ENGINE=InnoDB COLLATE = utf8_general_ci
+INSERT INTO t1 VALUES ();
+SELECT A FROM t1;
+A
+TRUE
+DROP TABLE t1;
+CREATE TABLE t1 (A BOOLEAN DEFAULT FALSE);
+SHOW CREATE TABLE t1;
+Table	Create Table
+t1	CREATE TABLE `t1` (
+  `A` BOOLEAN DEFAULT 'FALSE'
+) ENGINE=InnoDB COLLATE = utf8_general_ci
+INSERT INTO t1 VALUES ();
+SELECT A FROM t1;
+A
+FALSE
+DROP TABLE t1;
+CREATE TABLE t1 (A BOOLEAN DEFAULT NULL);
+SHOW CREATE TABLE t1;
+Table	Create Table
+t1	CREATE TABLE `t1` (
+  `A` BOOLEAN DEFAULT NULL
+) ENGINE=InnoDB COLLATE = utf8_general_ci
+INSERT INTO t1 VALUES ();
+SELECT A FROM t1;
+A
+NULL
+DROP TABLE t1;
+CREATE TABLE t1 (A BOOLEAN DEFAULT "");
+ERROR 42000: You have an error in your SQL syntax; check the manual that corresponds to your Drizzle server version for the right syntax to use near '"")' at line 1
+CREATE TABLE t1 (A BOOLEAN DEFAULT "false");
+ERROR 42000: You have an error in your SQL syntax; check the manual that corresponds to your Drizzle server version for the right syntax to use near '"false")' at line 1
+CREATE TABLE t1 (A BOOLEAN DEFAULT 0);
+ERROR 42000: You have an error in your SQL syntax; check the manual that corresponds to your Drizzle server version for the right syntax to use near '0)' at line 1
+CREATE TABLE t1 (A BOOLEAN DEFAULT 1);
+ERROR 42000: You have an error in your SQL syntax; check the manual that corresponds to your Drizzle server version for the right syntax to use near '1)' at line 1

=== added file 'tests/suite/bool_type/t/default.test'
--- tests/suite/bool_type/t/default.test	1970-01-01 00:00:00 +0000
+++ tests/suite/bool_type/t/default.test	2011-02-26 03:41:37 +0000
@@ -0,0 +1,26 @@
+CREATE TABLE t1 (A BOOLEAN DEFAULT TRUE);
+SHOW CREATE TABLE t1;
+INSERT INTO t1 VALUES ();
+SELECT A FROM t1;
+DROP TABLE t1;
+
+CREATE TABLE t1 (A BOOLEAN DEFAULT FALSE);
+SHOW CREATE TABLE t1;
+INSERT INTO t1 VALUES ();
+SELECT A FROM t1;
+DROP TABLE t1;
+
+CREATE TABLE t1 (A BOOLEAN DEFAULT NULL);
+SHOW CREATE TABLE t1;
+INSERT INTO t1 VALUES ();
+SELECT A FROM t1;
+DROP TABLE t1;
+
+--error ER_PARSE_ERROR
+CREATE TABLE t1 (A BOOLEAN DEFAULT "");
+--error ER_PARSE_ERROR
+CREATE TABLE t1 (A BOOLEAN DEFAULT "false");
+--error ER_PARSE_ERROR
+CREATE TABLE t1 (A BOOLEAN DEFAULT 0);
+--error ER_PARSE_ERROR
+CREATE TABLE t1 (A BOOLEAN DEFAULT 1);

=== modified file 'tests/suite/flush_tables/r/update_select.result'
--- tests/suite/flush_tables/r/update_select.result	2011-01-04 05:25:16 +0000
+++ tests/suite/flush_tables/r/update_select.result	2011-02-24 14:58:24 +0000
@@ -1,4 +1,5 @@
 CREATE TABLE t1 (a serial);
 select * from t1 for update;
 UNLOCK tables;
+ERROR HY000: No lock is held by this connection.
 DROP TABLE t1;

=== modified file 'tests/suite/flush_tables/t/update_select.test'
--- tests/suite/flush_tables/t/update_select.test	2011-01-04 05:25:16 +0000
+++ tests/suite/flush_tables/t/update_select.test	2011-02-24 14:58:24 +0000
@@ -17,6 +17,7 @@
 SEND select * from t1 for update;
 
 connection default;
+--error ER_NO_LOCK_HELD
 UNLOCK tables;
 
 DROP TABLE t1;

=== removed file 'tests/suite/mysql_compatibility/r/bool.result'
--- tests/suite/mysql_compatibility/r/bool.result	2011-02-03 00:52:25 +0000
+++ tests/suite/mysql_compatibility/r/bool.result	1970-01-01 00:00:00 +0000
@@ -1,18 +0,0 @@
-CREATE TABLE t1 (A BOOL, B VARCHAR(5));
-SHOW CREATE TABLE t1;
-Table	Create Table
-t1	CREATE TABLE `t1` (
-  `A` INT DEFAULT NULL,
-  `B` VARCHAR(5) COLLATE utf8_general_ci DEFAULT NULL
-) ENGINE=InnoDB COLLATE = utf8_general_ci
-INSERT INTO t1 VALUES (0, FALSE);
-INSERT INTO t1 VALUES (1, TRUE);
-INSERT INTO t1 VALUES (TRUE, TRUE);
-INSERT INTO t1 VALUES (FALSE, FALSE);
-SELECT A,B FROM t1;
-A	B
-0	0
-0	0
-1	1
-1	1
-DROP TABLE t1;

=== removed file 'tests/suite/mysql_compatibility/t/bool.test'
--- tests/suite/mysql_compatibility/t/bool.test	2011-02-03 00:52:25 +0000
+++ tests/suite/mysql_compatibility/t/bool.test	1970-01-01 00:00:00 +0000
@@ -1,11 +0,0 @@
-CREATE TABLE t1 (A BOOL, B VARCHAR(5));
-SHOW CREATE TABLE t1;
-
-INSERT INTO t1 VALUES (0, FALSE);
-INSERT INTO t1 VALUES (1, TRUE);
-INSERT INTO t1 VALUES (TRUE, TRUE);
-INSERT INTO t1 VALUES (FALSE, FALSE);
---sorted_result
-SELECT A,B FROM t1;
-
-DROP TABLE t1;

=== added file 'tests/suite/regression/r/682006.result'
--- tests/suite/regression/r/682006.result	1970-01-01 00:00:00 +0000
+++ tests/suite/regression/r/682006.result	2011-02-24 07:46:16 +0000
@@ -0,0 +1,7 @@
+create table t1 (a int);
+create table t2 (a int);
+flush tables with read lock;
+UNLOCK TABLES;
+ERROR HY000: No lock is held by this connection.
+unlock tables;
+drop table t1,t2;

=== added file 'tests/suite/regression/t/682006.test'
--- tests/suite/regression/t/682006.test	1970-01-01 00:00:00 +0000
+++ tests/suite/regression/t/682006.test	2011-02-24 07:46:16 +0000
@@ -0,0 +1,16 @@
+
+create table t1 (a int);
+create table t2 (a int);
+
+flush tables with read lock;
+
+connect (con1,localhost,root,,);
+connection con1;
+
+--error ER_NO_LOCK_HELD
+UNLOCK TABLES;
+
+connection default;
+disconnect con1;
+unlock tables;
+drop table t1,t2;

=== added file 'tests/t/concat.test'
--- tests/t/concat.test	1970-01-01 00:00:00 +0000
+++ tests/t/concat.test	2011-02-24 20:50:45 +0000
@@ -0,0 +1,5 @@
+# Test CONCAT operator
+SELECT "foo" || "dog";
+SELECT "1" || "2";
+SELECT 1 || 2;
+

=== modified file 'tests/t/flush.test'
--- tests/t/flush.test	2010-08-26 21:56:15 +0000
+++ tests/t/flush.test	2011-02-24 14:58:24 +0000
@@ -83,6 +83,7 @@
 send insert into t2 values(1);
 
 connection con1;
+--error ER_NO_LOCK_HELD
 unlock tables;
 disconnect con1;
 

